/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.team.vvc.shared.web.ui.context.ConfigurationContextBannerButton');

dojo.require("jazz.ui.ScrollableMenu");
dojo.require('com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem');
dojo.require('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog');
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm");
dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
dojo.require("com.ibm.team.vvc.shared.web.ui.VVC_Utils");

dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsClient");
dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");

	
dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");

(function ()
{
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	var VVC_Utils = com.ibm.team.vvc.shared.web.ui.VVC_Utils;
	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	var ChangesetsClient = com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsClient;
	
	/**
	 * ConfigurationContextBannerButton is a configuration context combo box to be placed in the Jazz UI bannner.
	 * It allows the user to choose a configuration context, and remembers the last 5 used. It also gives entry points
	 * into the VVC application by allowing the user to go the main VVC configuration explorer page or create workspaces
	 * or snapshots.
	 * 
	 * Pass "workspaceBasedOffSnapshotOnly" optional property to the first arguments object in the constructor to only 
	 * allow workspace creation off of a snapshot.
	 */
	dojo.declare('com.ibm.team.vvc.shared.web.ui.context.ConfigurationContextBannerButton', [dijit._Widget, dijit._Templated], 
		{
			templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/context/templates/ConfigurationContextBannerButton.html'),
			
			/**
			 * The index of the selected item, or the item itself.
			 * @field
			 * @type Number|Widget
			 */
			selected: null,
			redirectUriSelector: null,
			onContextChange: null,
			
			constructor: function (args)
			{
				if (args)
				{
					if (args.workspaceBasedOffSnapshotOnly)
						this._workspaceBasedOffSnapshotOnly = args.workspaceBasedOffSnapshotOnly;
					
					if (args.redirectUriSelector)
						this.redirectUriSelector = args.redirectUriSelector;
				}
				this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
			},

			postCreate: function() {
				this._initializing = true;
				this._initializeChangesetSubscribers();
				this._initializeMenuButton();
				this._initializeRecentlyUsed();
				this._initializeMoreMenuItem();
				this._initializeSelectedMenuItem();
				this._initializeManageConfigurations();
				this._initializeManageChangesets();
				
				this._toggleMenuItems();
				
				this._initializing = false;
				//Listen to update the widget when the context is completely initialized.
				this.subscribe("vvc.context_initialized",this._initializeSelectedMenuItem);
			},
			
			_initializeChangesetSubscribers: function()
			{
				dojo.subscribe("vvc.changeset_created", dojo.hitch(this, function() 
				{
					var changeset = arguments[0];
					if (this.redirectUriSelector != null && changeset != null && changeset.configurationUri != null && this.onContextChange != null)
					{
						var csConfiguration = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource({uri : changeset.configurationUri}); //not required to load yet
						this.onContextChange(csConfiguration);
					}
				} ));
			},
			
			addChild: function(widget, insertIndex){
				//add it to the menu
				this.connect(widget, "onClick", dojo.hitch(this, this._wasClicked, widget));
				this._menu.addChild.apply(this._menu, arguments);
				
				if (widget.configuration == null)
					return;
				
				ContextUtils.addConfigurationToRecentlyUsed(widget.configuration);
				
				//remove any menuItems that are no longer in the recently used list
				var menuItems = this._menu.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					var configuration = menuItem.configuration;
					if (configuration && !ContextUtils.isConfigurationInRecentlyUsed(configuration))
						this._menu.removeChild(menuItem);
				}
			},
			
			_initializeMenuButton: function()
			{
				this._menu = new jazz.ui.ScrollableMenu({
					selectable: true,
					onOpen: dojo.hitch(this, this._onMenuOpen),
					onClose: dojo.hitch(this, this._onMenuClose)
				});
				this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: this.messages.menuTitle}));
				
				this._menuPopup = new jazz.ui.MenuPopup({group: "jazz-app-BannerButtons", align: "right", menuProvider: dojo.hitch(this, this._menuProvider), menuClass:"jazz-app-Navbar-menu", injectCaret: true}, this._button);
			},
			
			_menuProvider:function()
			{
				return this._menu;
			},
			
			_setSelectedAttr: function(item) {
				if (typeof item == "number") {
					var widgets = this._menu.getChildren();
					if (item < widgets.length) {
						item = widgets[item];
					}
				}
				if (this.selected == item || !item.isFocusable()) return;
				
				if (item != null && item.domNode) {
					this._menu&&this._menu.setSelected(item);			
					
					if (item.iconClass) {
						if (this._previousIconClass)
							dojo.removeClass(this._icon, this._previousIconClass);
						dojo.addClass(this._icon, item.iconClass);
						this._previousIconClass = item.iconClass;
					} 
					else
						dojo.empty(this._icon);
					this._label.innerHTML = item.label;
				} 
				
				if (typeof this.selected !== "number" && this._initializing == false) this.onChange(item);
				this.selected = item;
			},

			_wasClicked: function(item) {
				this.set("selected", item);
			},

			_initializeSelectedMenuItem: function()
			{
				var currentContext = ContextUtils.getCurrentContext();
				
				if (currentContext == null)
					selectedMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
						label : this.messages.pleaseSelect,
						"class" : "com-ibm-team-vvc-context-menuItem",
					});
				else
				{
					selectedMenuItem = this._getMenuItemForConfiguration(currentContext);
					if (selectedMenuItem == null)
					{
						selectedMenuItem = this._constructMenuItem(currentContext);
						this.addChild(selectedMenuItem, 1);
					}
				}
				this._setSelectedAttr(selectedMenuItem);
			},
			
			_getMenuItemForConfiguration: function(configuration)
			{
				var menuItems = this._menu.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					var menuConfig = menuItem.configuration;
					if (menuConfig && configuration.uri == menuConfig.uri)
						return menuItem;
				}
				return null;
			},
			
			_onMenuOpen: function() {
				this._open = true;
				this._toggleActiveClass(true);
				this._menu.domNode.style.width = "";
				var menuWidth = dojo.position(this._menu.domNode).w;
				var thisWidth = dojo.position(this.domNode).w;
				if (menuWidth < thisWidth) {
					dojo.style(this._menu.domNode, 'width', thisWidth + 'px');
				}
				if (this.selected) {
					this._menu.setSelected(this.selected);
				}
			},

			_onMenuClose: function() {
				this._open = false;
				this._toggleActiveClass(false);
			},
			
			_initializeRecentlyUsed: function()
			{
				var recentlyUsed = ContextUtils.getRecentlyUsed();
				
				for(var i in recentlyUsed) {
					var recentConfig = recentlyUsed[i];
					this.addChild(this._constructMenuItem(recentConfig), 1);
				}
			},
			
			_initializeMoreMenuItem: function()
			{
				var self = this;
				var moreMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.more,
					isFocusable : function() {
						return false;
					},
					onClick: function() {
						var args = {};
						args.spaceUri = ContextUtils.getSpace();
							
						var picker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog(args);
						picker.onOk = function(configuration) {
							ContextUtils.setCurrentContext(configuration);
							self._initializeSelectedMenuItem();
							
							if (self.onContextChange) {
								self.onContextChange(configuration);
							}
						};
					}
				});				
				this.addChild(moreMenuItem);
			},
			
			_initializeManageChangesets: function()
			{
				var self = this;
				
				this._menu.addChild(new dijit.MenuSeparator());
				this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: this.messages.manageChangesets}));	
				
				this._createChangesetsMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.createChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-create-changeset",
					isFocusable : function() {
						return false;
					},
					onClick: function() {
						var form = new com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm({"configuration": ContextUtils.getCurrentContext().uri});
						form.show();
					},
					isApplicable: function(context) {
						if (context != null && context.isWorkspace())
							return true;
						return false;
					}
				});				
				this.addChild(this._createChangesetsMenuItem);
				
				this._renameChangesetMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.renameChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-rename-changeset",
					isFocusable : function() {
						return false;
					},
					onClick: dojo.hitch(this, function() {
						var name = prompt(this.messages.renamePrompt, this.selected.label);
						var changeset = this.selected.changeset;
						if (!name || name == changeset.title) {
							// if user has cancelled the input prompt
							return;
						}
						var deferred = ChangesetsClient.rename(changeset.getUri(), name);
						deferred.addCallback(this, dojo.hitch(this, function(){
							this.selected.setLabel(name);
							this._label.innerHTML = name;
							UIUtils.showPageMessage(this.messages.changesetRenamed, 5000);
						}));
						deferred.addErrback(this, dojo.hitch(this, function(error) {
							alert(dojo.string.substitute(this.messages.failedToRenameChangeset, [deferred.ioArgs.xhr.responseText]));
						}));
					}),
					isApplicable: function(context) {
						if (context != null && context.isChangeset())
							return true;
						return false;
					}
				});				
				this.addChild(this._renameChangesetMenuItem);
				
				this._completeChangesetMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.completeChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-complete-changeset",
					isFocusable : function() {
						return false;
					},
					onClick: dojo.hitch(this, function() {
						var confirmation = confirm(this.messages.completeConfirmation);
						if (!confirmation)
							return;
						var changeset = this.selected.changeset;
						var deferred = ChangesetsClient.complete(changeset.getUri());
						deferred.addCallback(this, dojo.hitch(this, function(){
							this.selected.setAttribute("iconClass", "com-ibm-team-vvc-context-completed-changeset");
							dojo.addClass(this._icon, "com-ibm-team-vvc-context-completed-changeset");
							this._toggleMenuItems();
							UIUtils.showPageMessage(this.messages.changesetCompleted, 5000);
						}));
						deferred.addErrback(this, dojo.hitch(this, function(error) {
							alert(dojo.string.substitute(this.messages.failedToCompleteChangeset, [deferred.ioArgs.xhr.responseText]));
						}));
					}),
					isApplicable: function(context) {
						if (context != null && context.isChangeset() && context.isMutable())
							return true;
						return false;
					}
				});
				this.addChild(this._completeChangesetMenuItem);
				
				this._commitChangesetMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.commitChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-commit-changeset",
					isFocusable : function() {
						return false;
					},
					onClick: dojo.hitch(this, function() {
						var confirmation = confirm(this.messages.commitConfirmation);
						if (!confirmation)
							return;

						var deferred = ChangesetsClient.commit(this.selected.changeset.getUri());
						deferred.addCallback(this, dojo.hitch(this, function(){
							alert(this.messages.changesetCommitted);
							this.onContextChange({uri: this.selected.configuration.getParentWorkspaceUri()});
						}));
						deferred.addErrback(this, dojo.hitch(this, function(error) {
							alert(dojo.string.substitute(this.messages.failedToCommitChangeset, [deferred.ioArgs.xhr.responseText]));
						}));
					}),
					isApplicable: function(context) {
						if (context != null && context.isChangeset())
							return true;
						return false;
					}
				});				
				this.addChild(this._commitChangesetMenuItem);
				
				this._discardChangesetsMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.discardChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-discard-changeset",
					isFocusable : function() {
						return false;
					},
					onClick: dojo.hitch(this, function() {
						var confirmation = confirm(this.messages.discardConfirmation);
						if (!confirmation)
							return;
							
						var deferred = ChangesetsClient.discard(this.selected.changeset.getUri());
						deferred.addCallback(this, dojo.hitch(this, function(){
							this.onContextChange({uri: this.selected.configuration.getParentWorkspaceUri()});
						}));
						deferred.addErrback(this, dojo.hitch(this, function(error) {
							alert(this.messages.failedToDiscardChangeset);
							alert(dojo.string.substitute(this.failedToDiscardChangeset, [deferred.ioArgs.xhr.responseText]));
						}));		
					}),
					isApplicable: function(context) {
						if (context != null && context.isChangeset() && context.isMutable())
							return true;
						return false;
					}
				});				
				this.addChild(this._discardChangesetsMenuItem);
				
				this._showChangesetsMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.showChangesets,
					isFocusable : function() {
						return false;
					},
					onClick: function() {
						self._showChangesets();
					},
				});				
				this.addChild(this._showChangesetsMenuItem);
			},
			
			_initializeManageConfigurations: function()
			{
				var self = this;
				
				this._menu.addChild(new dijit.MenuSeparator());
				this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: this.messages.manageConfigurations}));	
				
				this._newWorkspaceMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.newWorkspace,
					iconClass : "com-ibm-team-vvc-context-icon-new-workspace",
					isFocusable : function() {
						return false;
					},
					onClick: function() {
						self._newWorkspace();
					},
					isApplicable: dojo.hitch(this, function(context) {
						if (this._workspaceBasedOffSnapshotOnly)
						{
							if (context.isSnapshot())
								return true;
							else
								return false;
						}
						return true;
					})
				});		
				this.addChild(this._newWorkspaceMenuItem);
				
				this._newSnapshotMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.newSnapshot,
					iconClass : "com-ibm-team-vvc-context-icon-new-snapshot",
					isFocusable : function() {
						return false;
					},
					onClick: function() {
						self._newSnapshot();
					},
					isApplicable: function(context) {
						if (context.isWorkspace())
							return true;
						else
							return false;
					}
				});				
				this.addChild(this._newSnapshotMenuItem);
				
				this._openCurrentConfigurationMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.openCurrentConfiguration,
					iconClass : "com-ibm-team-vvc-context-icon-edit",
					isFocusable : function() {
						return false;
					},
					onClick: function() {
						self._openCurrentConfiguration();
					}
				});				
				this.addChild(this._openCurrentConfigurationMenuItem);
				
				var exploreMenuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.exploreConfigurations,
					isFocusable : function() {
						return false;
					},
					onClick: function() {
						self._showExplorer();
					}
				});	
				this.addChild(exploreMenuItem);
			},
			
			_toggleMenuItems: function()
			{
				var currentContext = ContextUtils.getCurrentContext();
				var menuItems = this._menu.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					if (menuItem.isApplicable != null)
					{
						if (currentContext != null && menuItem.isApplicable(currentContext))
							menuItem.setDisabled(false);
						else
							menuItem.setDisabled(true);
					}
				}
			},
			
			_constructMenuItem: function(configuration)
			{
				var menuItem = new com.ibm.team.vvc.shared.web.ui.widgets.HideableMenuItem({
					label : configuration.getTitle(),
					"class" : "com-ibm-team-vvc-context-menuItem",
					iconClass : ContextUtils.getConfigurationIconClass(configuration),
					configuration: configuration,
					changeset: ChangesetsClient.getCurrentChangeset(),
				});		
				return menuItem;
			},
			
			onChange: function(item)
			{
				if (item.configuration != null)
					ContextUtils.setCurrentContext(item.configuration);
				
				this._toggleMenuItems();
				
				if (this.onContextChange) {
					this.onContextChange(item.configuration);
				}
			},

			_onClick: function(event) {
				this._menuPopup.open();
			},

			_onMouseOver: function(event) {
				this._over = true;
				this._toggleActiveClass(true);
			},
			
			_onMouseOut: function(event) {
				this._over = false;
				this._toggleActiveClass(false);
			},
			
			_menuProvider: function()
			{
				return this._menu;
			},
			
			_toggleActiveClass: function(active) {
				dojo.toggleClass(this.domNode, 'active', active || this._open || this._over);
			},
			
			_openCurrentConfiguration: function()
			{
				if (ContextUtils.getVvcRootUri() != null)
				{
					var currentContext = ContextUtils.getCurrentContext();
					if (currentContext == null)
					{
						alert(this.noConfigurationSelected);
						return;
					}
					var newLocation = ContextUtils.getVvcRootUri() + "/web?" + this.getUriOptions();
					if (!currentContext.isMutable() || currentContext.getBase() != null)
						newLocation += "#action=SnapshotEditor";
					else
						newLocation += "#action=WorkspaceEditor";
					window.location.href = newLocation;
				}	
			},
			
			_newSnapshot:function()
			{
				if (ContextUtils.getVvcRootUri() != null)
				{
					var newLocation = ContextUtils.getVvcRootUri() + "/web?" + this.getUriOptions() + "#action=jazz.viewPage&id=SnapshotEditor&operation=create";
					window.location.href = newLocation;
				}
			},
			
			_newWorkspace:function()
			{
				if (ContextUtils.getVvcRootUri() != null)
				{
					var newLocation = ContextUtils.getVvcRootUri() + "/web?" + this.getUriOptions() + "#action=vvc.createWorkspace";
					window.location.href = newLocation;
				}
			},
			
			_showExplorer: function()
			{
				if (ContextUtils.getVvcRootUri() != null)
				{
					var newLocation = ContextUtils.getVvcRootUri() + "/web?" + this.getUriOptions() + "#action=vvc.showExplorer";
					window.location.href = newLocation;
				}
			},
			
			_showChangesets: function()
			{
				if (ContextUtils.getVvcRootUri() != null)
				{
					var newLocation = ContextUtils.getVvcRootUri() + "/web?" + this.getUriOptions(true) + "#action=vvc.showChangesets";
					window.location.href = newLocation;
				}
			},
			
			// sub class can override to include more additional parameters
			getUriOptions: function(returnOnlyWorkspace) 
			{
				var uriOptions = {};
				var currentContext = ContextUtils.getCurrentContext();
				var contextUri;
				uriOptions.spaceUri = ContextUtils.getSpace();
				
				if (currentContext != null){
					if (returnOnlyWorkspace && !currentContext.isMutable()){
						// want only workspace and snapshots are not mutable
						contextUri = currentContext.getParentWorkspaceUri();
					} else if (returnOnlyWorkspace && currentContext.isChangeset()){
						// want only workspace and changesets have a Change Set dimension 
						// and their owning workspace is their base
						contextUri = currentContext.getBase();
					} else {
						contextUri = currentContext.uri;
					}
					uriOptions.contextUri = contextUri;
				}
				if (this.redirectUriSelector)
						uriOptions.redirectUri =this.redirectUriSelector();
				return dojo.objectToQuery(uriOptions);
			},
	
		}
	);
})();