/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.shared.web.ui.history.HistoryTable");

dojo.require("dijit._Widget");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require('dojo.i18n');

dojo.require('com.ibm.team.vvc.shared.web.ui.VVC_Utils');
dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");

dojo.require('com.ibm.team.vvc.shared.web.ui.context.ContextUtils');

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.history", "HistoryMessages");

(function ()
{
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.history", "HistoryMessages");
	
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	var VVC_Utils = com.ibm.team.vvc.shared.web.ui.VVC_Utils;
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	
	var contextRoot = net.jazz.ajax._contextRoot;
	if (contextRoot.indexOf('jazz') != -1) contextRoot += '/jfs';
	
	
	dojo.declare("com.ibm.team.vvc.shared.web.ui.history.HistoryTable", [dijit._Widget], {
		
		constructor: function (args)
		{
			if (args)
			{
				if (args.displayErrorMessage)
					this.displayErrorMessage = args.displayErrorMessage;
				if (args.hideErrorMessage)
					this.hideErrorMessage = args.hideErrorMessage;
			}
		},
		
		postCreate: function(){			
			this._constructTable();
		},
	   		
		_constructTable: function() {
			if (!this._table)
			{
			    this._table = new jazz.ui.StyledTable({
					   columns: [
					      {label: "Name", width: "150px", sortable: false},
					      {label: "Date Created", width: "150px", sortable: false},
					      {label: "Creator", width: "150px", sortable: false},
					      {label: "Changeset", sortable: false}
					   ],
				});
				
				this._table.domNode.style.width = "100%";
				
				this._loadTable();
				
			    this.domNode.appendChild(this._table.domNode);
			    
			}
		},
		
		refresh: function()
		{
			UIUtils.showPageMessage("Refreshing explorer");
			this._loadTable();
			UIUtils.hidePageMessage();
		},
		
		_loadHistorySet: function()
		{
			var conceptUri = ContextUtils.getRequestParmValue("conceptUri");
			
			var configurationUri = ContextUtils.getContextUri();
			
			jazz.client.xhrGet({
				handleAs: 'json',
				sync: true,
				url: contextRoot + '/vvcshared/proxy', //$NON-NLS-1$
				"headers" : {
					"vvc_servicePath" : "/changesets/history"
				},
				content: {
					"conceptUri" : conceptUri,
					"configurationUri" : configurationUri,
				},

				load: dojo.hitch(this, function(result) {
					this.data = result;
				}),
				
				error: dojo.hitch(this, function(e) {
					console.log(e);
					if (this.displayErrorMessage)
						this.displayErrorMessage("Error: "+e.message);
					else
						alert('error '+e.message);
				})
			});
			
			return this.data;  
		},
		
		_loadTable: function()
		{
			var historySet = this._loadHistorySet();
			
			var conceptUri = ContextUtils.getRequestParmValue("conceptUri");
				
			var frag = document.createDocumentFragment();
			
			this._table.clearBody();

			var tr, td;
			for (var i = historySet.length-1; i >= 0; i--) {
				
				var versionLine = historySet[i];

				var versionedTitle = VVC_Utils.getDctermsDescriptionOnVersionedResource(versionLine.uri, conceptUri);
				if (i== (historySet.length-1)) //publish the latest version as the concept title
					dojo.publish("vvc.historyLastestVersionsTitle", [versionedTitle]);
				
				tr = document.createElement("tr");
				dojo.addClass(tr, "versionRow");
				if (i%2 == 1)
					dojo.addClass(tr, "oddRow")
				
				td = document.createElement("td");
				dojo.addClass(td, "historyCell")
				td.innerHTML = this._wrapAnchorAroundText(versionLine.uri, versionedTitle);
				tr.appendChild(td);
				
				td = document.createElement("td");
				dojo.addClass(td, "historyCell");
				td.innerHTML = VVC_Utils.formatDate(versionLine.creationTime);
				tr.appendChild(td);
				
				td = document.createElement("td");
				dojo.addClass(td, "historyCell");
				td.innerHTML = VVC_Utils.getFoafName(versionLine.creatorUri);
				tr.appendChild(td);
				
				td = document.createElement("td");
				var iconURL = this._getChangesetUrl(versionLine.changeset);
				var changesetText = "<img src="+"\""+iconURL+"\"" + " style=\"padding-top: 0px; padding-right: 4px;\"  height=\"16px\" width=\"16px\"\>"+versionLine.changeset.title;
				td.innerHTML = this._wrapAnchorAroundText(versionLine.changeset.uri, changesetText);
				tr.appendChild(td);
				
				frag.appendChild(tr);
			}
			
			this._table.tbody.appendChild(frag);
		},
		
		_getChangesetUrl: function(changeset){
			if (changeset.state == "COMMITTED")
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_committed.gif').uri;
			else if (changeset.state == "IN_PROGRESS")
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset.gif').uri;
			else if (changeset.state == "COMPLETED")
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_complete.gif').uri;
			else if (changeset.state == "SUSPENDED")
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_suspended.gif').uri;
			return "";
		},
		_wrapAnchorAroundText: function (uri, text)
		{
			return "<a href=\""+uri+"\">"+text+"<\a>";
		},
	});   
	})();
})();