/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.application.WebApplication");

	dojo.require("jazz.app._AjaxApplication");
	dojo.require("com.ibm.team.app.web.RepositoryApplication");
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.application.HelpDialog");
	dojo.require('dojox.timing');
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ConfigurationContextBannerButton");
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ConfigurationSpaceSelect");

	dojo.require("jazz.app.auth");
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");

	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var AdminClient            = com.ibm.team.repository.web.client.internal.AdminClient;

	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	var VVC_APP = "com.ibm.team.vvc.server.core";

	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.application.WebApplication", com.ibm.team.app.web.RepositoryApplication, { 

		name: "Configuration Management",
		
		testMode: false,
		
		iconSmall: dojo.moduleUrl(VVC_APP, "graphics/VVC_16.png"),
		iconLarge: dojo.moduleUrl(VVC_APP, "graphics/VVC_24.png"),
		autoRequestInitData: false,
		
		makeDummyCall: function() {
			var args = {
					handleAs: 'json',
					url: net.jazz.ajax.contextRoot() + '/queryvvc/vvcurl', //$NON-NLS-1$
					sync: true,
					load: function(response, ioArgs) {
						return response;
					}
				};
			var deferred =  jazz.client.xhrGet(args);
		},
		
		constructor: function(args) {
			
			if (this.testMode)
				this._addChangesetMenu();
				
			// Configure our application to use the pages we registered via extension
			// points, as well as the contributions we manually added
			this.config.usePageRegistry = true;
			
			// hide the search area
			this.ui._searchArea.style.display = "none";
			//this.config.showLoginLogout = false;
			
			/*
			 * when being redirected from another jazz app, it turns out that
			 * VVC client needs to explicitly do the OAuth dance if it hadn't
			 * already done so. We need to make sure that this OAuth dance
			 * happens before making a call to any other service. If we do not
			 * do this, all other calls fail at the 401 auth challenge.
			 */
			this.makeDummyCall();
						
			// TMA: Hack to avoid cross-domain blocks. We call AdminClient.getUserRoles first, since
			// any TeamServerClient.invokeService calls will fail when not authenticated
			// See WI: https://jazz.net/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/202484
			var srh = new ServiceResponseHandler(this, "_fetchUserRolesSuccess", "_fetchInitDataFailure");
			AdminClient.getUserRoles(srh);

		},
		
		_fetchUserRolesSuccess: function() {
			this.fetchInitData();
		},
		
		_fetchInitDataFailure: function() {
			this.inherited(arguments);
			this.ui.set("bannerTitle", this.getCurrentContext().name);
		},
		
		getCurrentContext: function() {
			return {name:"Configuration Management"};
		},
		
		_fetchInitDataSuccess: function(initializationData) {
			this.inherited(arguments);
			if (this.testMode)
				this._initializeContextSelectors();
		
		},
		_initializeContextSelectors: function()
		{
			var bannerButtons = this.ui.get("bannerRight");
			
			if (bannerButtons != null && bannerButtons != "")
			{
				this.contextBannerButton = new com.ibm.team.vvc.shared.web.ui.context.ConfigurationContextBannerButton({workspaceBasedOffSnapshotOnly : true});
				var pipeNode = document.createElement("span");
				dojo.addClass(pipeNode, 'pipe');
				pipeNode.innerHTML = "|";
				dojo.place(pipeNode, bannerButtons.domNode, "first");
				dojo.place(this.contextBannerButton.domNode, bannerButtons.domNode, "first");
				
//				this._setUpConfigurationSpaceSelect();
			}	
			
			
		},
		
		_setUpConfigurationSpaceSelect: function(){
			var bannerButtons = this.ui.get("bannerRight");
			
			if (bannerButtons != null && bannerButtons != "")
			{
				this.configurationSpaceSelect = new com.ibm.team.vvc.shared.web.ui.context.ConfigurationSpaceSelect();
				var pipeNode = document.createElement("span");
				dojo.addClass(pipeNode, 'pipe');
				pipeNode.innerHTML = "|";
				dojo.place(pipeNode, bannerButtons.domNode, "first");
				dojo.place(this.configurationSpaceSelect.domNode, bannerButtons.domNode, "first");
			}	
			dojo.subscribe("vvc.space_updated",  function(space){
				if (space)
					alert("The space switched to: '"+ space.title+"'");
				else
					alert("The space was null");
			});
		},
		
		_addChangesetMenu: function()
		{
			this.config.navbarItems.push({
				iconSrc: "", 
				title: "Change Sets", 
				label: "Change Sets",
				menuProvider: dojo.hitch(this, this._changesetMenuProvider)
			});
		},
		
		_changesetMenuProvider: function() {
			if (!this._changesetMenu) {
				this._changesetMenu = new jazz.app._ComponentMenu();
				this._changesetMenu.addItemByArgs({label: "Change Set Explorer Example", onClick: dojo.hitch(this, this._openChangesetExplorer)});
			}
			return this._changesetMenu;
		},
		
		_openChangesetExplorer: function()
		{
			PageUtil.showPage("vvc.showChangesets");
		},
		
		
		logout: function() {
			if (!net.jazz.ajax.ui.PlatformUI.getWorkbench().shouldVetoNavigation()) {
				var deferred = jazz.client.xhrPost({
					url: net.jazz.ajax._contextRoot + "/repository/logout",
					sync: true,
					load: function(res, ioArgs){
						jazz.client.xhrPost({
							url: ioArgs.xhr.getResponseHeader("X-rmps-logout-url"),
							headers: {
								"X-ibm-rmps-internal": "true"
							},
							sync: true,
							load: function(res, ioArgs){
								/*Required procedure to prevent the o auth interceptor from refreshing the auth information before the log out
								* process is done
								* Block popups first before reloading the page
								* 
								*IE onload behavior is different on onload way. So, we have to do it like that way 
								*/
								var frame = dojo.doc.createElement(dojo.isIE ? '<iframe onload="window.frameLoadCallback()">' : 'iframe');
						        frame.onload = window.frameLoadCallback = function() {
						           	/*Now, refresh the page after making sure that the interceptor popups will not effect the log out taking place.*/
									dojo.doc.location.reload();
						        };
								/*We MUST refresh after making sure that the logout dialog is closed.
								 * So, we do that in the unload method of its window*/
								/*Now, refresh the page after making sure that the interceptor popups will not effect the log out taking place.*/
								frame.src= net.jazz.ajax._dataServerRoot + "/auth/logout";
								frame.style.display= "none";
								
								dojo.body().appendChild(frame);
							}
						});
					}
				});
				deferred.addCallback(this, function (info) {
					window.location.reload(true);
				});
				deferred.addErrback(function(error) {
					var messageBox = new jazz.ui.MessageBox();
					messageBox.showMessage(error);
				});
			}
			return false;
		},

		uninitialize: function() {
			if (this._changesetMenu) this._changesetMenu.destroy();
			if (this._helpBox) this._helpBox.destroy();   
			if (this._helpMenu) this._helpMenu.destroy();
			if (this._navMenu) this._navMenu.destroy();
		}

	});
})();
