/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page.CompareViewer");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit.layout.ContentPane");
	dojo.require("dijit.layout.BorderContainer");
	dojo.require("dojo.parser");
	dojo.require("jazz.ui.Toolbar");
	dojo.require("jazz.ui.toolbar.Button");
	dojo.require("jazz.ui.MessageBox");
	dojo.require("jazz.ui.Breadcrumbs");
	
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	
	dojo.require("jazz.app.ua.PageHoverHelp");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "CompareViewerMessages");

	var ANCESTOR_PANE_ID = "ancestorPane";
	var LEFT_PANE_ID = "leftPane";
	var RIGHT_PANE_ID = "rightPane";
	var MERGE_PANE_ID = "mergePane";
	var MULTI_PANES_VIEWER_ID = "multiPanesViewer";
	var SINGLE_PANE_VIEWER_ID = "singlePaneViewer";
	var SINGLE_PANE_DIFF_PREFIX = "singlePaneDiff";
	var MULTI_PANES_DIFF_PREFIX = "multiPanesDiff";
	var SINGLE_PANE_ID = "singlePane";
	var LEFT_TITLE_ID = "leftTitleBar";
	var ANCESTOR_TITLE_ID = "ancestorTitleBar";
	var SYNC_SCROLL_ID = "syncScroll";
	var MIN_HEIGHT = 1; // can not be zero
	var scrollSkipCount = 0;
	var isMergeSession;
	
	var VVC_APP = "com.ibm.team.vvc.server.core";
	var ADD_ICON = dojo.moduleUrl(VVC_APP, "graphics/deltaAdd.gif").uri;
	var DELETE_ICON = dojo.moduleUrl(VVC_APP, "graphics/deltaDelete.gif").uri;
	var CHANGE_ICON = dojo.moduleUrl(VVC_APP, "graphics/deltaChange.gif").uri;
	var SYNC_ICON = dojo.moduleUrl(VVC_APP, "graphics/synced.gif").uri;
	var NO_SYNC_ICON = dojo.moduleUrl(VVC_APP, "graphics/no_synced.gif").uri;
	var ANCESTOR_CONFLICT_ICON = dojo.moduleUrl(VVC_APP, "graphics/resolve_ancestor_co.gif").uri;
	var DESCENDANT_CONFLICT_ICON = dojo.moduleUrl(VVC_APP, "graphics/resolve_left_co.gif").uri;
	var OTHER_DESCENDANT_CONFLICT_ICON = dojo.moduleUrl(VVC_APP, "graphics/resolve_right_co.gif").uri;

	function getSyncScroll() {
		return document.getElementById(SYNC_SCROLL_ID);
	}
	
	function scrolling(pane) {
		var sync = getSyncScroll();
		if (pane == undefined) return;
		if (sync && sync.checked == false) return;
		if (scrollSkipCount > 0) {
			scrollSkipCount--;
			return;
		}
		doScrolling(pane);
	}

	function doScrolling(pane) {
		var ancestorPane = document.getElementById(ANCESTOR_PANE_ID);
		var leftPane = document.getElementById(LEFT_PANE_ID);
		var rightPane = document.getElementById(RIGHT_PANE_ID);
		var mergePane = document.getElementById(MERGE_PANE_ID);
		if (leftPane == null || rightPane == null) return;
		if (ancestorPane != null) {
			if (leftPane == pane) {
				if (clickedHorizontalScrollBar == false) {
					rightPane.scrollTop = ancestorPane.scrollTop = leftPane.scrollTop;
				}
				rightPane.scrollLeft = ancestorPane.scrollLeft = leftPane.scrollLeft;
			} else if (rightPane == pane) {
				if (clickedHorizontalScrollBar == false) {
					leftPane.scrollTop = ancestorPane.scrollTop = rightPane.scrollTop;
				}
				leftPane.scrollLeft = ancestorPane.scrollLeft = rightPane.scrollLeft;
			} else if (ancestorPane == pane) {
				if (clickedHorizontalScrollBar == false) {
					leftPane.scrollTop = rightPane.scrollTop = ancestorPane.scrollTop;
					if (mergePane) {
						mergePane.scrollTop = ancestorPane.scrollTop;
					}
				}
				// change left and right setting only if ancestor pane has horizontal scroll bar
				if (ancestorPane.clientWidth < ancestorPane.scrollWidth) {
					leftPane.scrollLeft = rightPane.scrollLeft = ancestorPane.scrollLeft;
					if (mergePane) {
						mergePane.scrollLeft = ancestorPane.scrollLeft;
					}
				}
			} else if (mergePane == pane) {
				if (clickedHorizontalScrollBar == false) {
					leftPane.scrollTop = rightPane.scrollTop = mergePane.scrollTop;
					if (ancestorPane) {
						ancestorPane.scrollTop = mergePane.scrollTop;
					}
				}
				// change left and right setting only if ancestor pane has horizontal scroll bar
				if (mergePane.clientWidth < mergePane.scrollWidth) {
					leftPane.scrollLeft = rightPane.scrollLeft = mergePane.scrollLeft;
					if (ancestorPane) {
						ancestorPane.scrollLeft = mergePane.scrollLeft;
					}
				}
			}
		} else {
			if (leftPane == pane) {
				rightPane.scrollTop = leftPane.scrollTop;
				rightPane.scrollLeft = leftPane.scrollLeft;
			} else if (rightPane == pane) {
				leftPane.scrollTop = rightPane.scrollTop;
				leftPane.scrollLeft = rightPane.scrollLeft;
			}
		}
	}

	function showFirstDiff() {
		if (showSinglePaneFlag) { singlePaneDiffIndex = 0;  showSinglePaneDiff(0); }
		else { multiPanesDiffIndex = 0; showMultiPanesDiff(0); }
	}

	function showLastDiff() {
		if (lastDiffIndex == null) {
			var found, index = 0;
			lastDiffIndex = -1;
			do {
				found = false;
				for (var paneId = 0; paneId < 4 && found == false; paneId++) {
					var element = document.getElementById(getDiffId(index, paneId));
					if (element != undefined) {
						found = true;
					}
				}
				if (found) {
					lastDiffIndex = index;
					index++;
				}
			} while (found);
		}
		if (showSinglePaneFlag) { singlePaneDiffIndex = lastDiffIndex;  showSinglePaneDiff(0); }
		else { multiPanesDiffIndex = lastDiffIndex; showMultiPanesDiff(0); }
	}

	function showNextDiff() {
		if (showSinglePaneFlag) { showSinglePaneDiff(1); }
		else { showMultiPanesDiff(1); }
	}

	function showPrevDiff() {
		if (showSinglePaneFlag) { showSinglePaneDiff(-1); }
		else { showMultiPanesDiff(-1); }
	}

	var showSinglePaneFlag = false;
	var singlePaneHighlightElements = new Array();
	var singlePaneDiffIndex = -1;
	var multiPaneHighlightElements = new Array();
	var multiPanesDiffIndex = -1;
	var lastDiffIndex;
	var toolbars = new Array(4);
	var SELECTED_DELTA_ROW_STYLE = "2px solid red";

	function getPane(element) {
		if (element && element.offsetParent) {
			var parent = element.offsetParent;
			while (parent) {
				if (parent.id != null && parent.id.indexOf("Pane") != -1) {
					return parent;
				}
				parent = parent.offsetParent;
			}
		}
		return null;
	}

	function getElementHeight(element) {
		if (element.clientHeight == 0 && element.offsetParent != null) {
			// div section in IE may be 0. So, we need to use the parent td element height instead.
			return element.offsetParent.clientHeight;
		}
		return element.clientHeight;
	}

	function showSinglePaneDiff(increment) {
		singlePaneDiffIndex += increment;
		if (singlePaneDiffIndex < 0) singlePaneDiffIndex = 0;
		var pane = document.getElementById(SINGLE_PANE_ID);
		if (pane == undefined) return;

		// update the selection box
		var elementCount = 0;
		var yMin = pane.scrollHeight;
		var yMax = 0;
		for (var i = 0; i < 4; i++) {
			var element = document.getElementById(SINGLE_PANE_DIFF_PREFIX + singlePaneDiffIndex + "_" + i);
			if (element != undefined) {
				if (elementCount == 0) {
					// clear old selection first
					var e;
					do {
						e = singlePaneHighlightElements.pop();
						if (e != undefined) { e.style.border=""; }
					} while (e != undefined);
				}

				// high light selected elements
				elementCount++;
				singlePaneHighlightElements.push(element);
				element.style.border=SELECTED_DELTA_ROW_STYLE;
			}
		}

		for (var i = 0; i < singlePaneHighlightElements.length; i++) {
			var element = singlePaneHighlightElements[i];
			// compute the y range
			var yTop = getYPosition(element);
			var yBottom = yTop + getElementHeight(element);
			if (yTop < yMin) {
				yMin = yTop;
			}
			if (yBottom > yMax) {
				yMax = yBottom;
			}
		}
		var clientAreaBottom = pane.scrollTop + pane.clientHeight;
		var yOffset = getYPosition(pane);
		yMin -= yOffset;
		yMax -= yOffset;
		if (yMin > pane.scrollTop && yMax < clientAreaBottom) {
			// selected element is visible
		} else {
			pane.scrollTop = yMin - 2;
		}

		if (elementCount == 0) {
			singlePaneDiffIndex -= increment;
		}
	}
	
	function getDiffId(diffIndex, contributorId) {
		return MULTI_PANES_DIFF_PREFIX + diffIndex + "_" + contributorId;
	}

	function showMultiPanesDiff(increment) {
		multiPanesDiffIndex += increment;
		if (multiPanesDiffIndex < 0) multiPanesDiffIndex = 0;

		// scroll to selection
		var elementCount = 0;
		for (var paneIdex = 0; paneIdex < 4; paneIdex++) {
			var element = document.getElementById(getDiffId(multiPanesDiffIndex, paneIdex));
			if (element) {
				if (elementCount == 0) {
					// clear old selection first
					var e;
					do {
						e = multiPaneHighlightElements.pop();
						if (e != undefined) { e.style.border=""; }
					} while (e != undefined);
				}

				// highlight the new selection
				elementCount++;
				multiPaneHighlightElements.push(element);
				var toolbar = toolbars[paneIdex];
				if (paneIdex > 0 && toolbar && isPlaceHolderRow(element)) {
					element.style.border="2px solid #cccccc";
					toolbar.style.display = "none";
				} else {
					element.style.border=SELECTED_DELTA_ROW_STYLE;
					if (toolbar && isMergeSession) {
						toolbar.style.display = "block";
					}
				}
			}
		}

		// scroll to new selection
		scrollSkipCount = 6;
		for (var i = 0; i < multiPaneHighlightElements.length; i++) {
			var element = multiPaneHighlightElements[i];
			// scroll to selection
			var pane = getPane(element);
			if (pane != null) {
				var yOffset = getYPosition(pane);
				var yTop = getYPosition(element) - yOffset;
				var yBottom = yTop + getElementHeight(element);
				var clientAreaBottom = pane.scrollTop + pane.clientHeight;

				if (yTop > pane.scrollTop && yBottom < clientAreaBottom) {
					// selection is already visible
				} else {
					if ((yBottom - yTop) >= pane.clientHeight) {
						pane.scrollTop = yTop - 2;
					} else {
						pane.scrollTop = yTop - 20; // selection is about one line above top
					}
				}
				pane.scrollLeft = 0;
			}
		}

		if (elementCount == 0) {
			multiPanesDiffIndex -= increment;
		}
	}
	
	function isPlaceHolderRow(element) {
		var node = element;
		while (node) {
			if (node.tagName == "TR") {
				for (var i = 0; i < node.children.length; i++) {
					var child = node.children[i];
					if (child.tagName == "TD" && child.firstChild) {
						if (child.firstChild.tagName == "IMG") {
							// delta row always has icon
							return false;
						}
					}
				}
				return true;
			} else {
				node = node.parentNode;
			}
		}
		return false;
	}

	function showSinglePaneViewer() {
		if (showSinglePaneFlag) {
			return;
		}
		var v1 = document.getElementById(SINGLE_PANE_VIEWER_ID);
		var v2 = document.getElementById(MULTI_PANES_VIEWER_ID);
		showSinglePaneFlag = true;
		showAndHideViewer(v1, v2);
		updatePaneHeight();
		if (multiPanesDiffIndex >= 0) {
			singlePaneDiffIndex = multiPanesDiffIndex;
			showSinglePaneDiff(0);
		}
	}

	function showMultiPanesViewer() {
		if (showSinglePaneFlag == false) {
			return;
		}
		var v1 = document.getElementById(SINGLE_PANE_VIEWER_ID);
		var v2 = document.getElementById(MULTI_PANES_VIEWER_ID);
		showSinglePaneFlag = false;
		showAndHideViewer(v2, v1);
		updatePaneHeight();
		if (singlePaneDiffIndex >= 0) {
			multiPanesDiffIndex = singlePaneDiffIndex;
			showMultiPanesDiff(0);
		}
	}

	function showAndHideViewer(showViewer, hideViewer) {
		showViewer.style.position = "relative";
		showViewer.style.top = "0";
		showViewer.style.height = "80%";
		hideViewer.style.position = "absolute";
		hideViewer.style.top = "-1000000";
		hideViewer.style.height = "0";
	}

	function handleWindowResize() {
		scrollSkipCount = 6;
		updatePaneHeight();
	}

	function updatePaneHeight() {
		var ancestorPane = document.getElementById(ANCESTOR_PANE_ID);
		var leftPane = document.getElementById(LEFT_PANE_ID);
		var rightPane = document.getElementById(RIGHT_PANE_ID);
		var multiPaneViewer = document.getElementById(MULTI_PANES_VIEWER_ID);
		var singlePane = document.getElementById(SINGLE_PANE_ID);
		var singlePaneViewer = document.getElementById(SINGLE_PANE_VIEWER_ID);
		var yPosition, height, paneHeight;

		if (showSinglePaneFlag) {
			yPosition = getYPosition(singlePaneViewer);
			paneHeight = Math.round(100 - yPosition* 100 / document.body.clientHeight) + "%";
			singlePaneViewer.style.height =  paneHeight;
			singlePane.style.height = paneHeight;
		} else if (leftPane != undefined && rightPane != undefined) {
			yPosition = getYPosition(multiPaneViewer);
			height = Math.round(100 - yPosition * 100 / document.body.clientHeight);
			paneHeight = height + "%";
			multiPaneViewer.style.height = paneHeight;

			if (ancestorPane == undefined) { 
				leftPane.style.height = rightPane.style.height = paneHeight;
			} else {
				yPosition = getYPosition(ancestorPane);
				height = 100; //Math.round(100 - yPosition * 100 / document.body.clientHeight);
				paneHeight = height + "%";
				if (MIN_HEIGHT >= parseInt(leftPane.style.height.toString())) {
					setAncestorPaneHeight(ancestorPane, paneHeight);
				} else {
					var v = parseInt(ancestorPane.style.height.toString());
					leftPane.style.height = (height - v) + "%";
					rightPane.style.height = leftPane.style.height;
				}
			}
		}
	}
	
	function setAncestorPaneHeight(ancestorPane, paneHeight) {
		if (ancestorPane && paneHeight) {
			ancestorPane.style.height = paneHeight;
			var mergePane = document.getElementById(MERGE_PANE_ID);
			if (mergePane) {
				mergePane.style.height = paneHeight;
			}
		}
	} 

	// bind mouse events
	document.onmousemove = handleMouseMove;
	document.onmousedown = handleMouseDown;
	document.onmouseup = handleMouseUp;
	document.ondblclick = handleDoubleClick;
	document.onkeydown = handleKeyDown;

	var moveHorizontalSplitter = false; // splitter between left and right pane
	var moveAncestorMergeSplitter = false; // splitter between ancestor and merge pane
	var moveVerticalSplitter = false; // splitter between ancestor and left right pane
	var clickedAncestorTitleBar = false; // clicked splitter between left and right pane
	var clickedHorizontalScrollBar = false; // clicked splitter between ancestor and left right pane

	var downX, downY; // mouse down position
	var leftDragWidth, rightDragWidth, totalDragWidth; // saved left and right width values before dragging vertical splitter
	var ancestorDragWidth, mergeDragWidth, totalTopDragWidth; // saved ancestor and merge width values before dragging vertical splitter
	var ancestorDragHeight, leftDragHeight, totalDragHeight; // saved ancestor and left pane height values before dragging horizontal splitter
	var	yScale, xScale, xTopScale; // scales for converting pixels to percentage
	var previousAncestorHeight, previousLeftHeight; // saved height values from previous double clicked

	function getXPosition(element) {
		var xPosition = 0;
		if(element.offsetParent) {
			do { 
				xPosition += element.offsetLeft;
				if (!element.offsetParent)
					break;
				element = element.offsetParent;
			} while (true);
		} else if (element.x) {
			xPosition += element.x;
		}
		return xPosition;
	}

	function getYPosition(element) {
		var yPosition = 0;
		if (element.offsetParent) {
			do {
				yPosition += element.offsetTop;
				if(!element.offsetParent)
					break;
				element = element.offsetParent;
			} while (true);
		} else if (element.y) {
			yPosition += element.y;
		}
		return yPosition;
	}

	function getPercentage(value) {
		if (value && value.indexOf("%") != -1) {
			return parseInt(value);
		}
		return 0;
	}

	function getTotalPercentage(value1, value2) {
		return getPercentage(value1) + getPercentage(value2);
	}

	function handleMouseDown(e) {
		moveHorizontalSplitter = false;
		moveAncestorMergeSplitter = false;
		moveVerticalSplitter = false;
		clickedAncestorTitleBar = false;
		clickedHorizontalScrollBar = false;

		if (window.event != null) {
			downX = window.event.clientX + document.body.scrollLeft;
			downY = window.event.clientY + document.body.scrollTop;
		} else {
			downX = e.pageX;
			downY = e.pageY;
		}  
		if (downX < 0){downX = 0;}
		if (downY < 0){downY = 0;}

		var leftPane = document.getElementById(LEFT_PANE_ID);
		var rightPane = document.getElementById(RIGHT_PANE_ID);
		var ancestorPane = document.getElementById(ANCESTOR_PANE_ID);
		var mergePane = document.getElementById(MERGE_PANE_ID);
		var leftTitle = document.getElementById(LEFT_TITLE_ID);

		// calculate scale for horizontal drag between left and right pane
		leftDragWidth = leftPane.style.width.toString();
		rightDragWidth = rightPane.style.width.toString();
		totalDragWidth = getTotalPercentage(leftDragWidth, rightDragWidth);
		xScale = parseInt(totalDragWidth) * 1.0 / (leftPane.clientWidth + rightPane.clientWidth);

		// calculate scale for horizontal drag between ancestor and merge pane
		if (mergePane && ancestorPane) {
			ancestorDragWidth = ancestorPane.style.width.toString();
			mergeDragWidth = mergePane.style.width.toString();
			totalTopDragWidth = getTotalPercentage(ancestorDragWidth, mergeDragWidth);
			xTopScale = parseInt(totalTopDragWidth) * 1.0 / (ancestorPane.clientWidth + mergePane.clientWidth);
		}
		
		leftDragHeight = leftPane.style.height.toString()

		if (ancestorPane != null) {
			// calculate scale for vertical drag
			ancestorDragHeight = ancestorPane.style.height.toString();
			totalDragHeight = getTotalPercentage(ancestorDragHeight, leftDragHeight);
			yScale = parseInt(totalDragHeight) * 1.0 / (ancestorPane.clientHeight + leftPane.clientHeight);

			// test click on ancestor title bar
			var viewerTitle = document.getElementById(MULTI_PANES_VIEWER_ID);
			var ancestorPaneY = getYPosition(ancestorPane);
			var viewerTitleY = getYPosition(viewerTitle);
			if (downY > viewerTitleY && downY < ancestorPaneY) {
				clickedAncestorTitleBar = true;
			}

			// test click on vertical splitter
			var leftPaneY = getYPosition(leftPane);
			var leftTitleY = getYPosition(leftTitle);
			if (downY > leftTitleY && downY < leftPaneY) {
				document.onselectstart = function () { return false; };
				moveVerticalSplitter = true;
			}
			if (downY < leftTitleY && downY > leftTitleY - 20) {
				clickedHorizontalScrollBar = true;
			}
		}

		var maxX = getXPosition(rightPane);
		var minX = getXPosition(leftPane) + leftPane.offsetWidth;
		var minY = getYPosition(rightPane);
		var maxY = minY + (rightPane.clientHeight > 0? rightPane.clientHeight: leftPane.clientHeight);
		if (downX > minX && downX < maxX && downY > minY && downY < maxY) {
			document.onselectstart = function () { return false; }; // disable text selection during drag
			moveHorizontalSplitter = true;
		} else {
			if (downY > maxY && downY < (maxY + 20)) {
				clickedHorizontalScrollBar = true;
			} else if (ancestorPane && mergePane) {
				// test hitting splitter between ancestor and merge pane
				maxX = getXPosition(mergePane);
				minX = getXPosition(ancestorPane) + ancestorPane.offsetWidth;
				minY = getYPosition(mergePane);
				maxY = minY + (mergePane.clientHeight > 0? mergePane.clientHeight: ancestorPane.clientHeight);
				if (downX > minX && downX < maxX && downY > minY && downY < maxY) {
					document.onselectstart = function () { return false; }; // disable text selection during drag
					moveAncestorMergeSplitter = true;
				}
			}
		}

		if (moveVerticalSplitter || moveHorizontalSplitter || moveAncestorMergeSplitter) {
			// handle escape key while dragging the splitter
			document.onkeydown = function(evt) {
				var key;
				if (window.event) { key = window.event.keyCode;}
				else if (evt.which) { key = evt.which; }
				if (key == 27) { // escape key is pressed
					// restore width or height values of panes
					if (moveHorizontalSplitter) {
						leftPane.style.width = leftDragWidth;
						rightPane.style.width = rightDragWidth;
						var leftTitle = document.getElementById(LEFT_TITLE_ID);
						if (leftTitle) {
							leftTitle.style.width = leftPane.style.width;
						}
					} else if (mergePane && moveAncestorMergeSplitter) {
						ancestorPane.style.width = ancestorDragWidth;
						mergePane.style.width = mergeDragWidth;
						var ancestorTitle = document.getElementById(ANCESTOR_TITLE_ID);
						if (ancestorTitle) {
							ancestorTitle.style.width = ancestorPane.style.width;
						}
					} else if (moveVerticalSplitter && ancestorPane != null) {
						leftPane.style.height = leftDragHeight;
						rightPane.style.height = leftPane.style.height;
						setAncestorPaneHeight(ancestorPane, ancestorDragHeight);
					}
				}
				moveHorizontalSplitter = false;
				moveAncestorMergeSplitter = false;
				moveVerticalSplitter = false;
				document.onselectstart = null;
				document.onkeydown = null;
			};
		}
		return false; 
	}

	function handleDoubleClick(e) {
		handleMouseDown(e);
		var leftPane = document.getElementById(LEFT_PANE_ID);
		var rightPane = document.getElementById(RIGHT_PANE_ID);
		var ancestorPane = document.getElementById(ANCESTOR_PANE_ID);
		var mergePane = document.getElementById(MERGE_PANE_ID);

		// no processing for double click on horizontal splitter in two way compare
		if (ancestorPane == null && moveVerticalSplitter) {
			handleMouseUp(e);
			return;
		}

		var ancestorHeight = ancestorPane? parseInt(ancestorPane.style.height.toString()) : 0;
		var leftHeight = parseInt(leftPane.style.height.toString());

		if (ancestorPane && ancestorHeight <= MIN_HEIGHT && (moveVerticalSplitter || clickedAncestorTitleBar)) {
			var newAncestorHeight = previousAncestorHeight;
			if (newAncestorHeight == null || newAncestorHeight < 10) {
				newAncestorHeight = 10;
			}
			setAncestorPaneHeight(ancestorPane, newAncestorHeight + "%");
			var v = parseInt(leftPane.style.height.toString());
			leftPane.style.height = (v - newAncestorHeight) + "%"
			rightPane.style.height = leftPane.style.height;
		} else if (ancestorPane && leftHeight <= MIN_HEIGHT && (moveVerticalSplitter || clickedAncestorTitleBar)) {
			var newLeftHeight = previousLeftHeight;
			if (newLeftHeight == null || newLeftHeight < 10) {
				newLeftHeight = 10;
			}
			leftPane.style.height = rightPane.style.height = newLeftHeight + "%"
			setAncestorPaneHeight(ancestorPane, (ancestorHeight - newLeftHeight) + "%");		
		} else if (ancestorPane && moveVerticalSplitter) {
			var v = parseInt(leftPane.style.height.toString());
			previousAncestorHeight = ancestorHeight;
			setAncestorPaneHeight(ancestorPane, MIN_HEIGHT + "px");
			var v = parseInt(leftPane.style.height.toString());
			leftPane.style.height = (v + previousAncestorHeight) + "%"
			rightPane.style.height = leftPane.style.height;
		} else if (ancestorPane && clickedAncestorTitleBar) {
			var v = parseInt(leftPane.style.height.toString());
			previousLeftHeight = v;
			setAncestorPaneHeight(ancestorPane, (v + ancestorHeight) + "%");
			leftPane.style.height = rightPane.style.height = MIN_HEIGHT + "px";
		} else if (moveHorizontalSplitter) {
			leftPane.style.width = rightPane.style.width = "49%";
			var leftTitle = document.getElementById(LEFT_TITLE_ID);
			if (leftTitle) {
				leftTitle.style.width = leftPane.style.width;
			}
		} else if (moveAncestorMergeSplitter) {
			ancestorPane.style.width = mergePane.style.width = "49%";
			var ancestorTitle = document.getElementById(ANCESTOR_TITLE_ID);
			if (ancestorTitle) {
				ancestorTitle.style.width = ancestorPane.style.width;
			}
		}
		handleMouseUp(e);
	}

	function handleKeyDown(evt) {
		var key;
		if (window.event) { key = window.event.keyCode;}
		else if (evt.which) { key = evt.which; }

		if (key === 35) { // end key
			showLastDiff()
		} else if (key === 36) { // home key
			showFirstDiff();
		} else if (key === 38) { // arrow up key
			showPrevDiff();
		} else if (key === 40) { // arrow down key
			showNextDiff();
		}
	}
	
	function handleMouseUp(e) {
		moveHorizontalSplitter = false;
		moveAncestorMergeSplitter = false;
		moveVerticalSplitter = false;
		clickedAncestorTitleBar = false;
		clickedHorizontalScrollBar = false;
		document.onselectstart = null;
		document.onkeydown = handleKeyDown;
	}

	function handleMouseMove(e) {
		if (moveVerticalSplitter == false && moveHorizontalSplitter == false && moveAncestorMergeSplitter == false) {
			return true;
		}
		var mouseX = 0;
		var mouseY = 0;
		if (window.event != null) {
			mouseX = window.event.clientX + document.body.scrollLeft;
			mouseY = window.event.clientY + document.body.scrollTop;
		} else {
			mouseX = e.pageX;
			mouseY = e.pageY;
		}  
		if (mouseX < 30) {mouseX = 30;}
		if (mouseY < 0) {mouseY = 0;}  

		var leftPane = document.getElementById(LEFT_PANE_ID);
		var rightPane = document.getElementById(RIGHT_PANE_ID);
		var ancestorPane = document.getElementById(ANCESTOR_PANE_ID);
		var mergePane = document.getElementById(MERGE_PANE_ID);

		if (moveHorizontalSplitter) {
			var xOffset = Math.round((mouseX - downX) * xScale);
			var leftWidth = getPercentage(leftDragWidth);
			var rightWidth = getPercentage(rightDragWidth);
			var splitterWidth = 100 - totalDragWidth;

			if ((leftWidth + xOffset) <= 1) {
				leftPane.style.width = 1 + "px";
				rightPane.style.width = totalDragWidth + "%";
			} else if ((leftWidth + xOffset) >= totalDragWidth) {
				leftPane.style.width = totalDragWidth + "%";
				rightPane.style.width = 1 + "px";
			} else {
				var width = leftWidth + xOffset;
				leftPane.style.width = width + "%";
				rightPane.style.width = (totalDragWidth - width) + "%";
			}
			var leftTitle = document.getElementById(LEFT_TITLE_ID);
			leftTitle.style.width = leftPane.style.width;
		} else if (moveAncestorMergeSplitter && mergePane) {
			var xOffset = Math.round((mouseX - downX) * xTopScale);
			var leftWidth = getPercentage(ancestorDragWidth);
			var rightWidth = getPercentage(mergeDragWidth);
			var splitterWidth = 100 - totalTopDragWidth;

			if ((leftWidth + xOffset) <= 1) {
				ancestorPane.style.width = 1 + "px";
				mergePane.style.width = totalTopDragWidth + "%";
			} else if ((leftWidth + xOffset) >= totalDragWidth) {
				ancestorPane.style.width = totalTopDragWidth + "%";
				mergePane.style.width = 1 + "px";
			} else {
				var width = leftWidth + xOffset;
				ancestorPane.style.width = width + "%";
				mergePane.style.width = (totalTopDragWidth - width) + "%";
			}
			var ancestorTitle = document.getElementById(ANCESTOR_TITLE_ID);
			if (ancestorTitle) {
				ancestorTitle.style.width = ancestorPane.style.width;
			}
		} else if (moveVerticalSplitter) {
			var yOffset = Math.round((mouseY - downY) * yScale);
			var ancestorHeight = getPercentage(ancestorDragHeight);
			var leftHeight = getPercentage(leftDragHeight);

			if ((ancestorHeight + yOffset) <= MIN_HEIGHT) {
				setAncestorPaneHeight(ancestorPane, MIN_HEIGHT);
				leftPane.style.height = totalDragHeight + "%";
			} else if ((leftHeight - yOffset) <= MIN_HEIGHT) {
				setAncestorPaneHeight(ancestorPane, totalDragHeight + "%");
				leftPane.style.height = MIN_HEIGHT + "px";
			} else {
				setAncestorPaneHeight(ancestorPane, (ancestorHeight + yOffset) + "%");
				leftPane.style.height = (leftHeight - yOffset) + "%";
			}
			rightPane.style.height = leftPane.style.height;
		}
		return false;
	}

	function getTitle(id) {
		var element = document.getElementById(id);
		if (element) {
			return element.innerHTML;
		}
	}
	
	function setTitle(id, value) {
		var element = document.getElementById(id);
		if (element && value != null) {
			element.innerHTML = value;
		}
	}
	
	function setCompareViewerTitle(value) {
		setTitle("compareViewerTitle", value);
	}
	
	function setAncestorTitle(value) {
		setTitle("ancestorTitle", value);
	}
	
	function setLeftTitle(value) {
		setTitle("leftTitle", value);
	}
	
	function setRightTitle(value) {
		setTitle("rightTitle", value);
	}
	
	function setMergeTitle(value) {
		setTitle("mergeTitle", value);
	}
	
	// contributor names below must be same of names in ContributorType.java
	var ANCESTOR = "ancestor";
	var DESCENDANT = "descendant";
	var OTHER_DESCENDANT = "otherDescendant";
	var MERGE = "merge";
	
	// delta type names below must be same as names in DeltaType.java
	var ADD_DELTA = "AddDelta";
	var DELETE_DELTA = "DeleteDelta";
	var CHANGE_DELTA = "ChangeDelta";
	var OVERRIDE_ADD = "OverrideAddDelta"; // override added version in merge target
	
	// contributor id
	var ANCESTOR_ID = 0;
	var DESCENDANT_ID = 1;
	var OTHER_DESCENDANT_ID = 2;
	var MERGE_ID = 3;
	
	var LOCK_INFO = "resourceLockInfo";
	var COMPARE_PARTICIPANT = "compareParticipant";
	
	// --------------------------------------------------
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.CompareViewer", [dijit._Widget, dijit._Templated], {
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/page/templates/CompareViewer.html'),

		ancestorUri: null,
		descendantUri: null,
		otherDescendantUri: null,
		mergeUri: null,
		mergeOptions: null,
		diffCountInfo: null,
		operation: null,
		isMergeFlag: false,
		isThreeWayFlag: false,
		compareResult: null,
		ancestorDiffValues: [],
		descendantDiffValues: [],
		otherDescendantDiffValues: [],
		mergeDiffValues: [],
		deltaGroupItems: [], // array of groups of composite items
		
		constructor: function(workspace) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "CompareViewerMessages");
		},
		
		postCreate: function(){
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction(PageUtil.COMPARE_VIEWER_PAGE, this, "showPage");
			var onresizeHandle = dojo.connect(window, "onresize", this, this.onResize );
			this._unloadHandler = dojo.connect(window, "onbeforeunload", this, this.onBeforeUnload );
		},
		
		_initializeCSH: function() {
			if (!this._helpKeySet) {
				this._helpKeySet = true;
				var helpKey = this._getPageHoverHelpKey();
				this._ua = new jazz.app.ua.PageHoverHelp({uaKey: helpKey});
				this._pageHeader.appendChild(this._ua.domNode);
				this._ua.domNode.style.margin = "0px 0px 0px 5px";	
			}
	   	},
	   	
	   	_getPageHoverHelpKey: function() {
			var operation = PageUtil.getRequestParmValue("operation");
			if (operation == "rebase") {
				return "com.ibm.team.vvc.server.core.page_CompareViewer_Rebase001";
			} else if (operation == "accept") {
				return "com.ibm.team.vvc.server.core.page_CompareViewer_AcceptIncoming001";
			} else if (operation == "deliver") {
				return "com.ibm.team.vvc.server.core.page_CompareViewer_DeliverOutgoing001";
			} else if (operation == "compareWithParentSnapshot") {
				return "com.ibm.team.vvc.server.core.page_CompareViewer_CompareWithBaseSnapshot001";
			} else if (operation == "compareWithParentWorkspace") {
				return "com.ibm.team.vvc.server.core.page_CompareViewer_CompareWithParentWorkspace001";
			}
			return "com.ibm.team.vvc.server.core.page_CompareViewer001";
	   	},
	   	
		showPage: function() {
			this._initBreadcrumbs();
			
			this.ancestorUri = this.getRequestParmValue("ancestorUri");
			this.descendantUri = this.getRequestParmValue("descendantUri");
			this.otherDescendantUri = this.getRequestParmValue("otherDescendantUri");
			this.mergeUri = this.getRequestParmValue("mergeUri");
			this.mergeOptions = this.getRequestParmValue("mergeOptions");
			if (!this.mergeOptions) {
				this.mergeOptions = "lockDeltaResources";
			}
			this.operation = this.getRequestParmValue("operation");
			
			if (this.mergeUri != null) {
				this.isMergeFlag = true;
			} else {
				this.isMergeFlag = false;
				this.hideMergeUI();
			}
			
			if (this.ancestorUri && this.descendantUri && this.otherDescendantUri) {
				this.isThreeWayFlag = true;
				toolbars[0] = dojo.byId("ancestorToolBar");
				toolbars[1] = dojo.byId("leftToolBar");
				toolbars[2] = dojo.byId("rightToolBar");
				if (!this.isMergeFlag) {
					setMergeTitle(""); // clear merge pane title
					var pane = document.getElementById(ANCESTOR_PANE_ID);
					pane.style.width = "100%";
					pane = document.getElementById(MERGE_PANE_ID);
					pane.parentNode.removeChild(pane);
				}
			} else {
				if (this.ancestorUri == null) {
					this.ancestorUri = this.otherDescendantUri;
				}

				var ancestorPane = document.getElementById(ANCESTOR_PANE_ID);
				var mergePane = document.getElementById(MERGE_PANE_ID);
				var titleBar = document.getElementById("topTitleBar");
				var parent = ancestorPane.parentNode;
				if (this.isMergeFlag) {
					// use ancestor title and pane for merge result pane
					var title = getTitle("mergeTitle");
					setAncestorTitle(title);
					setMergeTitle(""); // clear merge pane title
					
					var pane = document.getElementById(ANCESTOR_PANE_ID);
					pane.style.width = "100%";
					pane = document.getElementById(MERGE_PANE_ID);
					pane.parentNode.removeChild(pane);
				} else {
					// show left and right panes only
					parent.removeChild(ancestorPane);
					parent.removeChild(mergePane);
					parent.removeChild(titleBar);
					var leftPane = document.getElementById(LEFT_PANE_ID);
					var rightPane = document.getElementById(RIGHT_PANE_ID);
					leftPane.style.height = rightPane.style.height = "99%";
				}
			}
			
			this.onResize();
			this._bindScroll(ANCESTOR_PANE_ID);
			this._bindScroll(LEFT_PANE_ID);
			this._bindScroll(RIGHT_PANE_ID);
			this._bindScroll(MERGE_PANE_ID);
			
			// send compare request to vvc server
			this.doCompare();
			
			var sync = document.getElementById(SYNC_SCROLL_ID);
			if (sync) {
				sync.checked = true;
			}
			
			this.updateTitles();
		},
		
		getRequestParmValue: function(paramName) {
			var value = PageUtil.getRequestParmValue(paramName);
			if (value != null) {
				value = unescape(value);
			}
			return value;
		},
		
		updateTitles: function() {
			var title, content;
			if (this.isThreeWayFlag) {
				// set ancestor title
				title = this.getTitle("ancestorTitle", ANCESTOR);
				setAncestorTitle(title);
				
				// set descendant title
				title = this.getTitle("descendantTitle", DESCENDANT);
				setLeftTitle(title);
				
				// set other descendant title
				title = this.getTitle("otherDescendantTitle", OTHER_DESCENDANT);
				setRightTitle(title);
				
				if (this.mergeUri != null) {
					isMergeSession = true;
					
					// set merge title
					title = this.getTitle("mergeTitle", MERGE);
					setMergeTitle(title);
				}
			} else {
				// set descendant title
				title = this.getTitle("descendantTitle", DESCENDANT);
				setLeftTitle(title);
				
				// set ancestor title
				title = this.getTitle("ancestorTitle", ANCESTOR);
				title = title;
				setRightTitle(title);
				
				if (this.mergeUri != null) {
					isMergeSession = true;
					
					// use ancestor pane title for merge pane title
					setMergeTitle("");
					title = this.getTitle("mergeTitle", MERGE);
					setAncestorTitle(title);
					
					// hide ancestor tool bar
					var toolbar = dojo.byId("ancestorToolBar");
					toolbar.style.display = "none";
				}
			}
			
			// update title
			var title = this.getTitle("title");
			if (title != null) {
				title = unescape(decodeURIComponent(title));
				setCompareViewerTitle(title);
			} else {
				// create title for known operations
				var operation = PageUtil.getRequestParmValue("operation");
				if (operation == "rebase") {
					title = this.messages.rebaseTitle;
				} else if (operation == "accept") {
					title = this.messages.acceptTitle;
				} else if (operation == "deliver") {
					title = this.messages.deliverTitle;
				} else if (operation == "compareWithParentSnapshot") {
					title = this.messages.compareWithParentSnapshotTitle;
				} else if (operation == "compareWithParentWorkspace") {
					title = this.messages.compareWithParentWorkspaceTitle;
				}
				if (title) {
					setCompareViewerTitle(title);
				}
			}
			
			this._initializeCSH();
		},
		
		getTitle: function(paramName, id) {
			var title = PageUtil.getRequestParmValue(paramName);
			if (title) {
				title = unescape(title);
			} else if (id) {
				var prefix = "";
				if (id == ANCESTOR) {
					prefix = this.messages.ancestorLabel;
				} else if (id == DESCENDANT) {
					prefix = this.messages.descendantLabel;
				} else if (id == OTHER_DESCENDANT) {
					prefix = this.messages.otherDescendantLabel;
				} else if (id == MERGE) {
					prefix = this.messages.mergeLabel;
				}
				
				var content = this.getContributorContent(id);
				if (content && content.contributorDisplayName) {
					title = content.contributorDisplayName;
				} else {
					if (id == ANCESTOR) {
						title = this.ancestorUri;
					} else if (id == DESCENDANT) {
						title = this.descendantUri;
					} else if (id == OTHER_DESCENDANT) {
						title = this.otherDescendantUri;
					} else if (id == MERGE) {
						title = this.mergeUri;
					}
					if (!title || title === "previous") {
						title = this.messages.none;
					}
					title = unescape(title);
				}
				title = prefix + title;
			}
			return title;
		},
		
		getConfigurationName: function(uri) {
			if (uri) {
				return unescape(uri);
			}
			return uri;
		},
		
		hideMergeUI: function() {
			this.setToolbarVisible(false);
		},
		
		showMergeUI: function() {
			this.setToolbarVisible(true);
		},
		
		setToolbarVisible: function(visible) {
			var ids = [ "ancestorToolBar", "leftToolBar", "rightToolBar", "mergeToolBar"];
			for (var id in ids) {
				var toolbar = dojo.byId(ids[id]);
				if (toolbar) {
					if (visible) {
						toolbar.style.display = "block";
					} else {
						toolbar.style.display = "none";
					}
				}
			}
		},
		
		doCompare: function() {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: this.messages.compareInProgress
			});
			
			var query = {};
			query.ancestorUri = this.ancestorUri;
			query.descendantUri = this.descendantUri;
			if (this.otherDescendantUri != null) {
				query.otherDescendantUri = this.otherDescendantUri;
			}
			if (this.mergeUri != null) {
				query.mergeUri = this.mergeUri;
			}
			if (this.mergeOptions != null) {
				query.mergeOptions = this.mergeOptions;
			}
			
			var clientUri = PageUtil.getRequestParmValue(PageUtil.CLIENT_INTERFACE);
			if (clientUri != null) {
				link += "&" + PageUtil.CLIENT_INTERFACE + "=" + clientUri;
			}
			
			var link = PageUtil.getVvcUri() + "/compare?" + dojo.objectToQuery(query);
			var args = {
				url: link,
				handleAs: "json",
				headers: {
					Accept: "application/json;application/rdf+json;text/rdf+json;text/json"
				},
				load: function(response, ioArgs) {
					return response;
				},
				error: dojo.hitch(this, function(error, ioArgs) {
					this.showMessage(PageUtil.createXhrErrorMessage(ioArgs));
					this.showMergeUI();
				})
			};
			var deferred = jazz.client.xhrGet(args);
			var self = this;
			deferred.addCallback(this, function (compareResult) {
				self.compareResult = compareResult;
				self.buildPanes();
			});
		},
		
		buildPanes: function() {
			// initialize diff count string
			var diffCountInfo = "";
			
			var content;
			if (this.ancestorUri && this.descendantUri && this.otherDescendantUri) {
				content = this.getPaneContent(ANCESTOR);
				this.refreshPane(ANCESTOR_PANE_ID, ANCESTOR_ID, content, this.ancestorDiffValues);
				content = this.getPaneContent(DESCENDANT);
				this.refreshPane(LEFT_PANE_ID, DESCENDANT_ID, content, this.descendantDiffValues);
				content = this.getPaneContent(OTHER_DESCENDANT);
				this.refreshPane(RIGHT_PANE_ID, OTHER_DESCENDANT_ID, content, this.otherDescendantDiffValues);
				
				// create diff count string
				diffCountInfo = this.getDiffCounts(ANCESTOR)
						+ this.getDiffCounts(DESCENDANT)
						+ this.getDiffCounts(OTHER_DESCENDANT);
				
			} else if (this.ancestorUri && this.descendantUri) {
				content = this.getPaneContent(ANCESTOR);
				this.refreshPane(RIGHT_PANE_ID, ANCESTOR_ID, content, this.ancestorDiffValues);
				content = this.getPaneContent(DESCENDANT);
				this.refreshPane(LEFT_PANE_ID, DESCENDANT_ID, content, this.descendantDiffValues);
				
				// create diff count string
				diffCountInfo = this.getDiffCounts(ANCESTOR) + this.getDiffCounts(DESCENDANT);
			}

			if (this.isMergeFlag) {
				content = this.getPaneContent(MERGE);
				if (this.isThreeWayFlag) {
					this.refreshPane(MERGE_PANE_ID, MERGE_ID, content, this.mergeDiffValues);
				} else {
					this.refreshPane(ANCESTOR_PANE_ID, MERGE_ID, content, this.mergeDiffValues);
				}
				this.acceptAllResolvableDifferences();
			}
			
			// update title with configuration names
			this.updateTitles();
			
			if (diffCountInfo && diffCountInfo.length > 0) {
				this.diffCountInfo = diffCountInfo;
				var msg = diffCountInfo;
				var msgType = jazz.ui.MessageBox.STATUS_TYPES.INFO;
				var lockInfo = this.compareResult[LOCK_INFO];
				if (lockInfo) {
					var lockedByOtherUserResources = lockInfo.lockedByOtherUserResources;
					
					if (lockedByOtherUserResources && lockedByOtherUserResources.length) {
						var resourceNames = "";
						for (var i = 0; i < lockedByOtherUserResources.length; i++) {
							var lockedResource = lockedByOtherUserResources[i];
							if (resourceNames.length > 0) {
								resourceNames += ", ";
							}
							var name = lockedResource.displayName;
							if (!name) {
								name = lockedResource.concept;
							}
							if (lockedResource.lockedByUser) {
								name = dojo.string.substitute(this.messages.lockedResourceInfo, [ name,  lockedResource.lockedByUser]);
							}
							resourceNames += name;
						}
						if (resourceNames.length > 0) {
							msgType = jazz.ui.MessageBox.STATUS_TYPES.ERROR;
							msg = dojo.string.substitute(this.messages.cannotChangeVersionLabel, [ resourceNames ]);
							this.hideMergeUI();
						}
					}
				}

				this.showMessage({
					type: msgType,
					summary: msg
				});
			} else {
				this.hideMessage();
			}
		},
		
		getPaneContent: function(contributorName) {
			if (this.compareResult) {
				var contData = this.compareResult[contributorName];
				if (contData) {
					return contData.resourceDiffs;
				}
			}
		},
		
		getContributorContent: function(contributorName) {
			if (this.compareResult) {
				return this.compareResult[contributorName];
			}
		},
		
		getDiffCounts: function(contributorName) {
			var contData = this.getContributorContent(contributorName);
			if (contData && contData.diffCounts) {
				var counts = contData.diffCounts;
				var info = "";
				if (counts.conflictCount && counts.conflictCount > 0) {
					info += dojo.string.substitute(this.messages.conflictCountLabel, [ counts.conflictCount ]);
				}
				if (counts.addCount && counts.addCount > 0) {
					if (info.length > 0) {
						info += ", ";
					}
					info += this.messages.addCountLabel + counts.addCount;
				}
				if (counts.deleteCount && counts.deleteCount > 0) {
					if (info.length > 0) {
						info += ", ";
					}
					info += this.messages.deleteCountLabel + counts.deleteCount;
				}
				if (counts.modifyCount && counts.modifyCount > 0) {
					if (info.length > 0) {
						info += ", ";
					}
					info += this.messages.modifyCountLabel + counts.modifyCount;
				}
				
				if (info.length > 0) {
					 info += ". ";
				} else if (contributorName != ANCESTOR) {
					info = this.messages.noChanges;
				}

				// add contributor name prefix
				if (contributorName != ANCESTOR) {
					var name = contData.contributorDisplayName;
					if (name && name.length > 60) {
						name = name.substring(0, 30) + "..." + name.substring(name.length - 30);
					}
					info = dojo.string.substitute(this.messages.contributorChangesLabel, [ name, info ]);
				}
				
				return info;

			}
			return "";
		},
		
		getDeltaIcon: function(deltaType, conflictType, contributorId) {
			var imagePath;
			if (contributorId == ANCESTOR_ID || contributorId == MERGE_ID) {
				if (conflictType) {
					imagePath = ANCESTOR_CONFLICT_ICON;
				}
			} else {
				if (deltaType == ADD_DELTA) {
					imagePath = ADD_ICON;
				} else if (deltaType == DELETE_DELTA) {
					imagePath = DELETE_ICON;
				} else if (deltaType == CHANGE_DELTA) {
					imagePath = CHANGE_ICON;
				}
			}
			return imagePath;
		},
		
		getDeltaStyleClass: function(deltaType, conflictType) {
			var styleClass;
			if (conflictType) {
				styleClass = "vvc-compare-conflictRow";
			} else if (deltaType == ADD_DELTA) {
				styleClass = "vvc-compare-deltaAddRow";
			} else if (deltaType == DELETE_DELTA) {
				styleClass = "vvc-compare-deltaDeleteRow";
			} else if (deltaType == CHANGE_DELTA) {
				styleClass = "vvc-compare-deltaChangeRow";
			}
			return styleClass;
		},
		
		getItemText: function(item) {
			// DEBUG start
//			var text = "";
//			if (item.spacer) {
//				text = "PlaceHolder: ";
//			}
//			return item.version + " | displayName= " + item.displayName + " | delta=" + item.deltaType + " | groupId=" + item.groupId + " concept=" + item.concept;
			// DEBUG end
			
			if (item.spacer) {
				// no text for place holder entry
				return;
			}
			var text = item.displayName;
			if (!text) {
				text = item.version;
			}
			return text;
		},
		
		refreshPane: function(paneId, contributorId, paneContent, diffValues) {
			var pane = dojo.byId(paneId);
			if (paneContent && pane && pane.firstChild) {
				for (var index = 0; index < pane.childNodes.length; index++) {
					var child = pane.childNodes[index];
					var tag = child.tagName;
					if (tag === "TABLE") {
						var table = child;
						for (var i = table.rows.length - 1; i >= 0; i--) {
							table.deleteRow(i);
						}
						
						// fill table with compare result
						var diffIndex = 0;
						var item, tr, td;
						var groupItems = [];
						for (var index = 0; index < paneContent.length; index++) {
							item = paneContent[index];
							
							// check for composite delta group
							if (item.groupId) {
								if (item.deltaType) {
									var group = groupItems[item.groupId];
									if (!group) {
										group = [];
										groupItems[item.groupId] = group;
									}
									group.push(item);
								}

								if (item.displayName == null) {
									// hidden resource or composite delta
									continue;
								}
							}
							
							tr = dojo.create("tr", null, table);
							tr.style.height = "20px";

							var conflictType = item.conflictType;
							var deltaType = item.deltaType;
							var iconContainer = dojo.create("td", null, tr);
							if (deltaType) {
								var imagePath = this.getDeltaIcon(deltaType, conflictType, contributorId);
								if (imagePath) {
									// show delta icon
							        var img = dojo.create("IMG", null, iconContainer);
							        img.src = imagePath;
							        iconContainer.title = this.getDeltaTooltip(deltaType, conflictType, contributorId);
								}
									
								// row content
								td = dojo.create("td", null, tr);
								var div = dojo.create("div", null, td);
								var styleClass = this.getDeltaStyleClass(deltaType, conflictType);
								if (styleClass) {
									dojo.addClass(div, styleClass);
								}
								
								diffValues[diffIndex] = { item: item, div: div, 
										iconContainer: iconContainer, imagePath: imagePath };
								div.id = getDiffId(diffIndex++, contributorId);

								var text = this.getItemText(item);
								
								// DEBUG spacer start
//								if (text == null && item.spacer) {
//									text = "spacer: " + item.concept;
//								}
								// DEBUG spacer end
								
								if (text) {
									var tn = dojo.doc.createTextNode(text)
									div.appendChild(tn);
								} else {
									// place holder entry for the corresponding add or delete entry
									dojo.create("br", null, div);
								}
								
								dojo.connect(tr, "onclick", this, this.onSelectedDelta);
								
							} else {
								// row content
								td = dojo.create("td", null, tr);
								var text = this.getItemText(item);
								
								// DEBUG spacer start
//								if (text == null && item.spacer) {
//									text = "spacer: " + item.concept;
//								}
								// DEBUG spacer end
								
								if (text) {
									var tn = dojo.doc.createTextNode(text)
									if (item.ancestorDiffType) {
										var div = dojo.create("div", null, td);
										div.appendChild(tn);
										div.id = getDiffId(diffIndex++, contributorId);
									} else {
										td.appendChild(tn);
									}
								} else {
									// place holder entry for the corresponding add or delete entry
									var div = dojo.create("div", null, td);
									if (item.ancestorDiffType) {
										diffValues[diffIndex] = { item: item, div: div, iconContainer: iconContainer };
										div.id =  getDiffId(diffIndex++, contributorId);
										dojo.create("br", null, div);
									}
								}
							}
						}
						
						// add composite delta group to array
						if (groupItems.length > 0) {
							this.deltaGroupItems[contributorId] = groupItems;
						}
						
						return table;
					}
				}
			}
		},
		
		onSelectedDelta: function(e) {
			var tr = e.target;
			if (tr && tr.id && tr.id.indexOf(MULTI_PANES_DIFF_PREFIX) == 0) {
				var id = tr.id.substring(MULTI_PANES_DIFF_PREFIX.length);
				var end = id.indexOf ("_");
				if (end != -1) {
					this.showDiffInfo();
					id = id.substring(0, end);
					var index = parseInt(id);
					showMultiPanesDiff(index - multiPanesDiffIndex);
				}
			}
		},

		_bindScroll: function(paneId) {
			var pane = dojo.byId(paneId);
			if (pane) {
				dojo.connect(pane, "onscroll", this, this.onScroll);
			}
		},

		onScroll: function(event) {
			scrolling(event.target);
		},

		onResize: function() {
			var viewer = dojo.byId(MULTI_PANES_VIEWER_ID);
			var yPosition = getYPosition(viewer);
			var paneHeight = document.body.clientHeight - yPosition - 100 + "px";

			viewer.style.height = paneHeight;
		},
		
		onBeforeUnload: function(e) {
			// discard the uncomplete merge session
			this.discardMergeResult();
		},
		
		discardMergeResult: function() {
			// clear unload handler
			if (this._unloadHandler) {
				dojo.disconnect(this._unloadHandler);
				this._unloadHandler = null;
			}
			
			if (!this.mergeUri || !this.compareResult) {
				// no clean up for compare session
				return;
			}
			
			var lockInfo = this.compareResult[LOCK_INFO];
			if (!lockInfo) {
				// no resource locks to clean up
				return;
			}
			this.hideMergeUI();
			
			var self = this;
			var data = {};
			data.operation = "disposeMergeSession";
			
			// pass compare session info back to the merge service
			data[LOCK_INFO] = lockInfo;
			data[COMPARE_PARTICIPANT] = this.compareResult[COMPARE_PARTICIPANT];
			
			// dispose locked resources
			var uri = PageUtil.getVvcUri() + "/compare?mergeUri=" + this.mergeUri;
			var args = {
				url: uri,
				headers: {
					"Accept" : "application/json",
					"Content-Type" : "application/json"
				},
				postData: dojo.toJson(data),
			};

			var deferred = jazz.client.xhrPost(args);
		},
		
		gotoNextDiff: function() {
			this.showDiffInfo();
			showNextDiff();
		},
		gotoPreviousDiff: function() {
			this.showDiffInfo();
			showPrevDiff();
		},
		
		gotoFirstDiff: function() {
			this.showDiffInfo();
			showFirstDiff();
		},
		
		gotoLastDiff: function() {
			this.showDiffInfo();
			showLastDiff();
		},
		
		showDiffInfo: function() {
			if (this.diffCountInfo) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: this.diffCountInfo
				});	
			}
		},
		
		toggleSyncScroll: function() {
			var sync = getSyncScroll();
			if (sync) {
				if (sync.checked === null) {
					sync.checked = true;
				}
			}
			sync.checked = !sync.checked;
			if (this.syncButton && this.syncButton._img && this.syncButton._img.src) {
				if (sync.checked == true) {
					this.syncButton._img.src = SYNC_ICON;
				} else {
					this.syncButton._img.src = NO_SYNC_ICON;
				}
			}
		},
		
		getDeltaTooltip: function(deltaType, conflictType, contributorId, isMergePane) {
			if (conflictType) {
				if (contributorId == MERGE_ID || (contributorId == ANCESTOR_ID && isMergePane)) {
					return this.messages.acceptAncestor;
				}
				if (contributorId == ANCESTOR_ID) {
					if (conflictType == "AddAddConflict") {
						return this.messages.addAddConflict;
					} else if (conflictType == "DeleteDeleteConflict") {
						return this.messages.deleteDeleteConflict;
					} else if (conflictType == "ChangeChangeConflict") {
						return this.messages.changeChangeConflict;
					} else if (conflictType == "ChangeDeleteConflict") {
						return this.messages.changeDeleteConflict;
					}
				}
			}
			
			if (isMergePane) {
				if (deltaType == ADD_DELTA) {
					return this.messages.acceptAdd;
				} else if (deltaType == DELETE_DELTA) {
					return this.messages.acceptDelete;
				} else if (deltaType == CHANGE_DELTA) {
					return this.messages.acceptChange;
				}
			}
			
			if (deltaType == ADD_DELTA) {
				return this.messages.added;
			} else if (deltaType == DELETE_DELTA) {
				return this.messages.deleted;
			} else if (deltaType == CHANGE_DELTA) {
				return this.messages.changed;
			}
		},
		
		acceptAncestorDiff: function() {
			this.acceptSelectedEntry(this.ancestorDiffValues, ANCESTOR_ID);
		},
		
		acceptLeftDiff: function() {
			this.acceptSelectedEntry(this.descendantDiffValues, DESCENDANT_ID);
		},
		
		acceptRightDiff: function() {
			if (this.isThreeWayFlag) {
				this.acceptSelectedEntry(this.otherDescendantDiffValues, OTHER_DESCENDANT_ID);
			} else {
				this.acceptSelectedEntry(this.ancestorDiffValues, ANCESTOR_ID);
			}
		},
		
		acceptSelectedEntry: function (contributorValues, contributorId) {
			if (multiPanesDiffIndex >= 0) {
				var contributorEntry = contributorValues[multiPanesDiffIndex];
				if (contributorEntry) {
					var mergeEntry = this.mergeDiffValues[multiPanesDiffIndex];
					if (contributorEntry.item && contributorEntry.item.deltaType) {
						this.updateMergeEntry(mergeEntry, contributorEntry, contributorId);
						
						// also accept delta with the same group id
						var groupId = mergeEntry.item.groupId;
						if (!groupId) {
							return;
						}
						for (var i = 0; i < contributorValues.length; i++) {
							if (i == multiPanesDiffIndex) {
								continue; // already process this entry
							}
							var otherContributorEntry = contributorValues[i];
							if (otherContributorEntry) {
								var otherMergeEntry = this.mergeDiffValues[i];
								if (otherContributorEntry.item
										&& otherContributorEntry.item.groupId == groupId
										&& otherContributorEntry.item.deltaType) {
									this.updateMergeEntry(otherMergeEntry, otherContributorEntry, contributorId);
								}
							}
						}
					}
				}
			} else if (contributorValues.length > 0) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: this.messages.noDeltaSelected
				});
			}
		},
		
		updateMergeEntry: function(mergeEntry, contributorEntry, contributorId) {
			// update text in merge entry
			var div = mergeEntry.div;
			if (div.firstChild) {
				div.removeChild(div.firstChild);
			}
			var text = this.getItemText(contributorEntry.item);
			if (text) {
				var tn = dojo.doc.createTextNode(text)
				div.appendChild(tn);
			} else {
				// place holder entry for the corresponding add or delete entry
				dojo.create("br", null, div);
			}
			mergeEntry.item = contributorEntry.item;
			
			// update icon in merge entry
			var iconContainer = mergeEntry.iconContainer;
			if (iconContainer.firstChild) {
				iconContainer.removeChild(iconContainer.firstChild);
			}
			if (contributorEntry.imagePath) {
				// show delta icon
		        var img = dojo.create("IMG", null, iconContainer);
		        img.src = contributorEntry.imagePath;
		        var item = contributorEntry.item;
		        if (item) {
			        iconContainer.title = this.getDeltaTooltip(item.deltaType, item.conflictType, contributorId, true);
		        }
			}
			
			// update the accepted contributor id
			mergeEntry.contributorId = contributorId;
		},
		
		revertMergeSession: function() {
			this.buildPanes();
			showMultiPanesDiff(0);
		},
		
		reload: function() {
			this.discardMergeResult();
			window.location.reload();
		},
		
		acceptAllResolvableDifferences: function() {
			this.showDiffInfo();
			for (var i = 0; i < this.mergeDiffValues.length; i++) {
				var mergeEntry = this.mergeDiffValues[i];
				if (mergeEntry && mergeEntry.item && mergeEntry.item.conflictType) {
					var ct = mergeEntry.item.conflictType;
					if (ct != "AddAddConflict" && ct != "DeleteDeleteConflict" && ct != "ChangeChangeConflict") {
						continue;
					}
				}
				var entry = this.descendantDiffValues[i];
				if (entry && entry.item && entry.item.deltaType) {
					this.updateMergeEntry(mergeEntry, entry, DESCENDANT_ID);
				} else {
					entry = this.otherDescendantDiffValues[i];
					if (entry && entry.item && entry.item.deltaType) {
						this.updateMergeEntry(mergeEntry, entry, OTHER_DESCENDANT_ID);
					}
				}
			}
		},
		
		commitMergeSession: function() {
			var deltaContainer = [];
			var targetContributorId = ANCESTOR_ID;
			var contributorDiffValues = [];
			if (this.mergeUri == this.descendantUri) {
				targetContributorId = DESCENDANT_ID;
				contributorDiffValues = this.descendantDiffValues;
			} else if (this.mergeUri == this.otherDescendantUri) {
				targetContributorId = OTHER_DESCENDANT_ID;
				contributorDiffValues = this.otherDescendantDiffValues;
			}
			
			for (var mergeIndex = 0; mergeIndex < this.mergeDiffValues.length; mergeIndex++) {
				var entry = this.mergeDiffValues[mergeIndex];
				if (!entry.contributorId) {
					entry.contributorId = ANCESTOR_ID;
				}
				
				var contributorId = entry.contributorId;
				if (contributorId != targetContributorId) {
					var item = entry.item;
			        var deltaType = item.deltaType;
			        var contributorItem;
		        	if (mergeIndex < contributorDiffValues.length && contributorDiffValues[mergeIndex]) {
			        	contributorItem = contributorDiffValues[mergeIndex].item;
		        	}
			        	
			        if (contributorId == ANCESTOR_ID) {
			        	if (contributorItem) {
				        	deltaType = contributorItem.deltaType;
				        	if (deltaType == ADD_DELTA || deltaType == DELETE_DELTA) {
				        		// reverse the deltas in other contributor
					        	this.addReverseDeltas(contributorItem, contributorId, deltaContainer);
							} else if (deltaType == CHANGE_DELTA) {
								// reverse the deltas in merge contributor
								var changeFromDeltas = this.getCompositeDeltas(contributorItem, targetContributorId);
								var changeToDeltas = this.getCompositeDeltas(item, contributorId);
								var compositeDeltas = this.createChangeDeltas(changeFromDeltas, changeToDeltas);
								this.pushAllItems(compositeDeltas, deltaContainer);
							}
			        	}
			        } else if (deltaType == ADD_DELTA) {
			        	this.addCompositeDeltas(item, contributorId, deltaContainer);
			        	if (item.conflictType) {
			        		if (contributorItem && contributorItem.deltaType == ADD_DELTA) {
			        			// add add conflict
			        			var compositeDeltas = this.getCompositeDeltas(contributorItem, targetContributorId);
			        			var overrideAddDeltas = this.duplicateDeltas(compositeDeltas, OVERRIDE_ADD);
			        			this.pushAllItems(overrideAddDeltas, deltaContainer);
			        		}
			        	}
					} else if (deltaType == DELETE_DELTA) {
						this.addCompositeDeltas(item, contributorId, deltaContainer);
					} else if (deltaType == CHANGE_DELTA) {
						var compositeDeltas = this.getCompositeDeltas(item, contributorId);
						if (item.conflictType) {
							// only need to adjust the deltas when merge target is not same as ancestor
							if (targetContributorId != ANCESTOR_ID) {
								if (contributorItem.deltaType == DELETE_DELTA) {
									// for change delete conflict, create add deltas
									for (var index = 0; index < compositeDeltas.length; index++) {
										var addDelta = {};
										dojo.mixin(addDelta, compositeDeltas[index]);
										addDelta.deltaType = ADD_DELTA;
										compositeDeltas[index] = addDelta;
									}
								} else if (contributorItem.deltaType == CHANGE_DELTA) {
									// for change change conflict, set target contributor mapping as ancestor mapping
									var changeFromDeltas = this.getCompositeDeltas(contributorItem, targetContributorId);
									compositeDeltas = this.createChangeDeltas(changeFromDeltas, compositeDeltas);
								}
							}
						}

						this.pushAllItems(compositeDeltas, deltaContainer);
					}
				}
			}
			
			if (deltaContainer.length > 0) {
				var self = this;
				var data = {};
				data.deltas = deltaContainer;
				if (this.operation == "rebase") {
					data.rebaseParentConfiguration = this.otherDescendantUri;
				}
				data.operation = this.operation;
				data.ancestorUri = this.ancestorUri;
				data.descendantUri = this.descendantUri;
				data.otherDescendantUri = this.otherDescendantUri;

				// pass compare session info back to the merge service
				if (this.compareResult[LOCK_INFO]) {
					data[LOCK_INFO] = this.compareResult[LOCK_INFO];
				}
				if (this.compareResult[COMPARE_PARTICIPANT]) {
					data[COMPARE_PARTICIPANT] = this.compareResult[COMPARE_PARTICIPANT];
				}
				
				var uri = PageUtil.getVvcUri() + "/compare?mergeUri=" + this.mergeUri;
				var args = {
						url: uri,
						headers: {
							"Accept" : "application/json",
							"Content-Type" : "application/json"
						},
						sync: true,
						handleAs: "json",
						postData: dojo.toJson(data),
						load: function(response, ioArgs) {
							// merge successfully committed
							if (self.operation == "rebase") {
								history.go(-2); // skip over the rebase page
							} else {
								self.postCommitMergeSession();
							}
						},
						error: dojo.hitch(this, function(error, ioArgs) {
							this.showMergeUI();
							this.showMessage(PageUtil.createXhrErrorMessage(ioArgs));
						})
				};
				
				// show message before committing the  merge result
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: this.messages.commitInProgress
				});
				this.hideMergeUI();
				
				var deferred = jazz.client.xhrPost(args);
			} else {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.noChangesToMerge
				});
			}
		},
		
		postCommitMergeSession: function() {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: this.messages.committed
			});
			if (this._unloadHandler) {
				dojo.disconnect(this._unloadHandler);
			}
		},
		
		cancelMergeSession: function() {
			this.discardMergeResult();
			history.back();
		},
		
		duplicateDeltas: function(deltas, deltaType) {
			var deltaArray = [];
			for (var i = 0; i < deltas.length; i++) {
				var item = {};
				dojo.mixin(item, deltas[i]);
				if (deltaType) {
					item.deltaType = deltaType;
				}
				deltaArray.push(item);
			}
			return deltaArray;
		},
		
		pushAllItems: function (items, itemContainer) {
			for (var index = 0; index < items.length; index++) {
				itemContainer.push(items[index]);
			}
		},
		
		getCompositeDeltas: function(deltaItem, contributorId) {
			var deltaArray = [];
			deltaArray.push(deltaItem);
			
			var groupId = deltaItem.groupId;
			var groupItems = this.deltaGroupItems[contributorId];
			if (groupId && groupItems) {
				var group = groupItems[groupId];
				if (group) {
					for (var i = 0; i < group.length; i++) {
						if (group[i] != deltaItem) {
							deltaArray.push(group[i]);
						}
					}
				}
			}
			return deltaArray;
		},
		
		addCompositeDeltas: function(deltaItem, contributorId, deltaContainer) {
			var compositeDeltas = this.getCompositeDeltas(deltaItem, contributorId);
			for (var i = 0; i < compositeDeltas.length; i++) {
				deltaContainer.push(compositeDeltas[i]);
			}
		},
		
		findDelta: function(concept, deltas) {
			for (var i = 0; i < deltas.length; i++) {
				var delta = deltas[i];
				if (delta.concept == concept) {
					return delta;
				}
			}
		},
		
		createChangeDeltas: function(changeFromDeltas, changeToDeltas) {
			var changeDeltas = [];
			for (var i = 0; i < changeToDeltas.length; i++) {
				var toDelta = changeToDeltas[i];
				var fromDelta = this.findDelta(toDelta.concept, changeFromDeltas);
				if (fromDelta && toDelta) {
					var chageDelta = {};
					dojo.mixin(chageDelta, toDelta);
					chageDelta.ancestorMappingUri = fromDelta.mappingUri;
					changeDeltas.push(chageDelta);
				}
			}
			return changeDeltas;
		},
		
		addReverseDeltas: function(deltaItem, contributorId, deltaContainer) {
			var compositeDeltas = this.getCompositeDeltas(deltaItem, contributorId);
			for (var i = 0; i < compositeDeltas.length; i++) {
				var delta = compositeDeltas[i];
				var reverseDelta = this.createReverseDelta(delta)
				deltaContainer.push(reverseDelta);
			}
		},
		
		createReverseDelta: function(deltaItem) {
			var reverseDeltaItem = {};
			dojo.mixin(reverseDeltaItem, deltaItem);
			
			if (deltaItem.deltaType == ADD_DELTA) {
				reverseDeltaItem.deltaType = DELETE_DELTA;
			} else if (deltaItem.deltaType == DELETE_DELTA) {
				reverseDeltaItem.deltaType = ADD_DELTA;
			}
			return reverseDeltaItem;
		},
		
		showMessage: function(args) {
			UIUtils.hidePageMessage();
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},
		
		_initBreadcrumbs: function()
		{
			if (!this._breadcrumbs)
			{
				var explorerPageUri = PageUtil.createLink(PageUtil.SHOW_EXPLORER);
				var parts = [];
				var segment = PageUtil.getClientAppBreadcrumbSegment();
				if (segment != null) {
					parts.push(segment);
				}
				
				var segment = {
						label: this.messages.configurations, 
						href : explorerPageUri
					};
				parts.push(segment);
				
				var operation = PageUtil.getRequestParmValue("operation");
				var breadcrumb = PageUtil.getRequestParmValue("breadcrumb");
				if (breadcrumb) {
					breadcrumb = decodeURIComponent(unescape(breadcrumb));
				} else {
					breadcrumb = this.messages.workspace;
				}
				
				if (operation == "rebase") {
					segment = {
							label: breadcrumb, 
							href : 'javascript:history.go(-2)'
						};
					parts.push(segment);
					
					segment = {
							label: this.messages.rebase, 
							href : 'javascript:history.back()'
						};
					parts.push(segment);
				} else if (operation == "deliver" || operation == "accept"
						|| operation == "compareWithPreviousSnapshot"
						|| operation == "compareWithParentSnapshot"
						|| operation == "compareWithParentWorkspace") {
					segment = {
							label: breadcrumb, 
							href : 'javascript:history.back()'
						};
					parts.push(segment);
				}
				this._breadcrumbs = new jazz.ui.Breadcrumbs({segments: parts});
				this._breadcrumbs.placeAt(dojo.byId("breadcrumbs"));
			}
		},
		
		uninitialize: function() {
			if (this.dialog) this.dialog.destroy();   
		}
		
	}) // end of CompareViewer

	
})();
