/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page.ResourceMappingViewer");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;

	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "ResourceMappingViewerMessages");

	var VVC_APP = "com.ibm.team.vvc.server.core";

	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.ResourceMappingViewer", [dijit._Widget, dijit._Templated], {

		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl(VVC_APP, "ui/internal/page/templates/ResourceMappingViewer.html"),

		_configurationUri: null,
		_lazyStart: false,
		_mappings: null,

		constructor: function(args) {
		if (args) {
			if (args.configurationUri) {
				this._configurationUri = args.configurationUri;
				this._lazyStart = args.lazyStart;
			}
		}
		this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "ResourceMappingViewerMessages");
	},

	postCreate: function() {
		if (!this._lazyStart) {
			this.load();
		}
	},

	load: function(resizableContainer) {
		// show loading message
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: this.messages.loading
		});
		
		var link = PageUtil.getVvcUri() + "/repository/configuration/resourceVersions?configurationUri=" + escape(this._configurationUri);
		var clientUri = PageUtil.getRequestParmValue(PageUtil.CLIENT_INTERFACE);
		if (clientUri != null) {
			link += "&" + PageUtil.CLIENT_INTERFACE + "=" + clientUri;
		}
		var args = {
			url: link,
			headers: {
			"Accept" : "application/json;application/rdf+json;text/rdf+json;text/json"
		},
		handleAs: "json"
		};

		// build resource version table
		var deferred = jazz.client.xhrGet(args);
		deferred.addCallback(this, function (param) {
			var tableContent = this._buildContentForResourceTab(param, resizableContainer);
		});
		deferred.addErrback(this, function(error) {
			this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
		});
	},
	
	_buildContentForResourceTab: function(mappings, resizableContainer) {
		this._mappings = mappings;
		
		// remove loading message
		this.hideMessage();

		// create table header
		var columnData = [ { label: this.messages.name, sortable: true },
		                   { label: this.messages.version, width: "100px", sortable: true },
		                   { label: this.messages.id, sortable: true }];
		var container = this._containerNode;
		if (this._resourceMappingTable) {
			container.removeChild(this._resourceMappingTable);
		}

		// create table contents
		this.tableContainer = dojo.create("div", "overflow: auto; white-space: nowrap; display: inline-block", container);
		var self = this;
		table = new jazz.ui.StyledTable({
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
			columns: columnData,
			sortBy: 0,
			sortByAscending: false,
			handleSort: function() {
				this.clearBody();
				self._sort(mappings, table);
				self._fillTable(mappings, table);
			}
		});
		this._resourceMappingTable = table;
		this.tableContainer.appendChild(table.domNode);
		table.domNode.style.width = "100%";
		
		dojo.addClass(this.tableContainer, "vvc-editor-divTableContentStyle");
		if (resizableContainer) {
			resizableContainer.addResizableChildElement(this.tableContainer);
		}
		
		// change version ui from version number
		if (mappings && mappings.length) {
			for (var i = 0; i < mappings.length; i++) {
				var entry = mappings[i];
				
				// initialize name field
				entry.name = entry.displayName;
				if (!entry.name) {
					entry.name = entry.concept;
				}
				
				// change version to last segment
				entry.versionId = entry.version;
				if (entry.versionId) {
					var index = entry.versionId.lastIndexOf("/");
					if (index != -1) {
						entry.versionId = entry.versionId.substring(index + 1);
					}
				}
				
				// initialize resource id field
				entry.resourceId = entry.concept;
				if (entry.concept && entry.concept.indexOf("/vvc/dependencies/space/") !== -1) {
					entry.displayName = this.messages.dependencies;
					entry.resourceId = entry.version;
				}
				if (entry.resourceId) {
					var index = entry.resourceId.lastIndexOf("/");
					if (index != -1) {
						entry.resourceId = entry.resourceId.substring(index + 1);
					}
				}
			}
		}
		
		this._sort(mappings, table);
		this._fillTable(mappings, table);
	},
	
	_fillTable: function(mappings, table) {
		// Now add the rows
		if (mappings && mappings.length) {
			var tr, td, label;
			for (var i = 0; i < mappings.length; i++){
				var entry = mappings[i];
				tr = document.createElement("tr");
				
				// name column
				td = document.createElement("td");
				var name = entry.displayName;
				if (name) {
					this.setLinkField(td, name, entry.displayNameHref);
				} else {
					this.setLinkField(td, entry.concept);
				}
				tr.appendChild(td);
				
				// version column
				td = document.createElement("td");
				this.setLinkField(td, entry.versionId);
				tr.appendChild(td);
				
				// resource id column
				td = document.createElement("td");
				this.setLinkField(td, entry.resourceId);
				tr.appendChild(td);
				
				table.tbody.appendChild(tr);
			}
		}

		return table;
	},
	
	setLinkField: function(field, value, uri) {
		if (field) {
			if (field.firstChild) {
				field.removeChild(field.firstChild);
			}

			if (uri) {
				var link = dojo.create("a");
				link.href = uri;
				link.innerHTML = jazz.util.html.escape(value);
				field.appendChild(link);
			} else {
				var node = dojo.doc.createTextNode(value);
				field.appendChild(node);
			}
		}
	},
	
	_sort: function(configResources, table) {
		var tableColumn = this._getTableColumn;
		var mutableIndex = this._mutableIndex;
		var f = function(a, b) {
			var c1 = tableColumn(a, table.sortBy, mutableIndex);
			var c2 = tableColumn(b, table.sortBy, mutableIndex);
			var r = 0;
			if (c1 && c2) {
				if (c1 < c2) {
					r = -1; 
				} else if (c1 > c2) {
					r = 1;
				}
				if (table.sortByAscending) {
					r = r * -1;
				}
			} else if (c1) {
				return -1;
			} else {
				return 1;
			}
			return r;
		};
		configResources.sort(f);
	},

	_getTableColumn: function(mapping, columnIndex, index) {
		if (columnIndex === 0) {
			return mapping.name;
		}
		if (columnIndex === 1) {
			return mapping.versionId;
		}
		if (columnIndex === 2) {
			return mapping.resourceId;
		}
	},
	
	showMessage: function(args) {
		if (args && this._messageBoxContainer) {
			args["closable"] = true;
			if (!this.messageBox) {
				this.messageBox = new jazz.ui.MessageBox();
				this._messageBoxContainer.appendChild(this.messageBox.domNode);
			}
			this.messageBox.showMessage(args);
			this._messageBoxContainer.style.display = "";
		}
	},
	
	hideMessage: function() {
		if (this._messageBoxContainer) {
			this._messageBoxContainer.style.display = "none";
		}
	},

	})   
})();
