/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page.WorkspaceEditor");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit.InlineEditBox");
	dojo.require("jazz.util.html");
	dojo.require("jazz.ui.MessageBox");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.snapshots.SnapshotsViewer");
	dojo.require("com.ibm.team.vvc.server.core.ui.snapshots.WorkspacesAndSnapshotsViewer");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationSpace");
	
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("jazz.app.ua.PageHoverHelp");
	dojo.require("jazz.app.ua.TextHoverHelp");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.widgets.tab.TabManager");
	
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;

	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.page._EditorWidget");
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.page.ResourceMappingViewer");

	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "WorkspaceEditorMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "WorkspaceEditorMessages");
	
	var NON_BASE_SNAPSHOT = "noSnapshot";
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.WorkspaceEditor", [com.ibm.team.vvc.server.core.ui.internal.page._EditorWidget, dijit._Templated], {

		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/page/templates/WorkspaceEditor.html'),

		_editableTitle: null,
		_workspace: null,
		_parentUri: null,
		_spaceUri: null,
		_parentConfig: null,
		_snapshotComboBox: null,
		_isCreateOperation: false,
		_createCompleted: false,

		constructor: function() {
			this.messages = messages;
			this.workspaceIconUrl = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/workspace.gif').uri;
		},
	
		postCreate: function() {
			// Register the page action
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction(PageUtil.WORKSPACE_PAGE, this, "showPage");
			ActionRegistry.registerAction("vvc.createWorkspace", this, "showCreateWorkspace");
			var onresizeHandle = dojo.connect(window, "onresize", this, this.onResize);
			
			this._initializeTabs();
		},
		
		_initializeTabs: function()
		{
			this._tabManager = new com.ibm.team.vvc.shared.web.ui.widgets.tab.TabManager();
			var class_selectedTab= "Selected";
			this._tabManager.registerTab(this._tabOverview, this.overview_content, class_selectedTab);
			this._tabManager.registerTab(this._tabDimensions, this.dimensions_content, class_selectedTab);
			this._tabManager.registerTab(this._tabDependencies, this.dependencies_content, class_selectedTab, dojo.hitch(this, function() { this._dependenciesViewer.load(this);}));
			this._tabManager.registerTab(this._tabSnapshots, this.snapshots_content, class_selectedTab, dojo.hitch(this, function() { this._snapshotsViewer.load(this);}));
			this._tabManager.registerTab(this._tabWorkspaces, this.workspaces_content, class_selectedTab, dojo.hitch(this, function() { this._workspacesViewer.load(this);}));
			this._tabManager.registerTab(this._tabChangesets, this.changesets_content, class_selectedTab, dojo.hitch(this, function() { this._changesetsViewer.load(this);}));
			this._tabManager.registerTab(this._tabResources, this.resources_content, class_selectedTab, dojo.hitch(this, function() { this._resourceMappingTable.load(this);}));
		},
		
		_initializeCSH:function()
		{
			if (this._ua != null)
				this._pageHeader.removeChild(this._ua.domNode);
			if (this._isCreateOperation)
				cshKey =  "com.ibm.team.vvc.server.core.page_WorkspaceEditorCreate001";
			else
				cshKey =  "com.ibm.team.vvc.server.core.page_WorkspaceEditorEdit001";	
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: cshKey});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
			
		},
		
		showCreateWorkspace: function()
		{
			this._isCreateOperation = true;
			this._renderPage();
		},
		
		showPage: function() {
			this._isCreateOperation = PageUtil.isCreate();
			this._renderPage();
		},
		
		_clearFields: function() {
			this._parentUri = null;
			this.setParentWorkspaceField("");
			this.setParentSnapshotField("");
			this.hideMessage();
		},
		
		_hideNonCreationTabs: function()
		{
			this._tabDependencies.style.display = "none";
			this._tabSnapshots.style.display = "none";
			this._tabWorkspaces.style.display = "none";
			this._tabChangesets.style.display = "none";
			this._tabResources.style.display = "none";
			if (!ContextUtils.getContextUri()) //hide dimension tab if this is a root level create
				this._tabDimensions.style.display = "none";
		},
		
		_renderPage: function() {
			this._initBreadcrumbs();
			this._initializeCSH();
			this._clearFields();
			this.enableActions();
			
			var context = ContextUtils.getContextUri();
			
			if (this._isCreateOperation) {
				this._hideNonCreationTabs();
				this._tabManager.showTab(this._tabOverview);

				if (context) {
					// load parent configuration
					this._parentUri = unescape(context);
					var parentConfig = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
					var link = PageUtil.getVvcUri() + "/repository/configuration?uri=" + this._parentUri;
					parentConfig.load(link, dojo.hitch(this, this.onLoadError));
					this._parentConfig = parentConfig;
					
					// set parent workspace and snapshot fields 
					var parentName = parentConfig.getTitle();
					var dimensions = [];
					var dimension;
					var separator = "\n"; // separator use by repository service
					if (parentConfig.isMutable()) {
						this.setParentWorkspaceField(parentName, this._parentUri);
						this.createParentSnapshotChoices();
						
						dimension = {};
						dimension.value = messages.oneGreaterThanDimension + separator + parentConfig.getTitle();
						dimensions.push(dimension);
						
					} else {
						this.setParentWorkspaceField(parentConfig.getParentWorkspaceName(), parentConfig.getParentWorkspaceUri());
						this.setParentSnapshotField(parentName, this._parentUri);
						
						dimension = {};
						dimension.value = "TimeOrder" + separator + parentName;
						dimensions.push(dimension);
						
						dimension = {};
						dimension.value = messages.oneGreaterThanDimension + separator + parentConfig.getParentWorkspaceName();
						dimensions.push(dimension);
					}
					
					// build content of dimension tab
					this._buildDimensionTab(dimensions);
				}
				
				// create editable workspace title
				this.createEditableTitleNode(this._titleNode, this.messages.newWorkspaceName, this.messages.clickToEdit);
			    
				this.connect(this._descriptionField, "onchange", function () {
					this.setDirty(true);
				});
			} else {
				var uri = unescape(context);
				var workspace = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
				var link = PageUtil.getVvcUri() + "/repository/configuration?uri=" + uri;
				var deferred = workspace.load(link, dojo.hitch(this, this.onLoadError));
				deferred.addCallback(this, function (param) {
					this._workspace = workspace;
					this.refresh();
				});
				deferred.addErrback(this, function(error) {
					this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
					this.enableActions();
				});
			}
			
			var spaceTitle = null;
			if (this._workspace) {
				spaceTitle = this._workspace.getSpaceName();
			}
			var spaceUri = PageUtil.getRequestParmValue("spaceUri");
			if (spaceUri && !spaceTitle) 
			{
				this._spaceUri = unescape(spaceUri);
				var space = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationSpace();
				space.load(this._spaceUri);
				spaceTitle = space.getTitle();
			} else if (this._isCreateOperation) {
				spaceTitle = this.messages.globalSpaceTitle; // by default TODO: Allow user to select the configuration space with the picker
			}
			this.setSpaceField(spaceTitle);
			
			this._initButtons();
		},

		onSelectDimensions: function() {
			alert(messages.notSupportedYet)
		},
		
		onSelectDependancies: function() {
			var args = {};
			if (this._workspace != null) {
				args.contextUri = this._workspace.uri;
			}
			PageUtil.showPage(PageUtil.ADD_DEPENDENCY_PAGE, dojo.objectToQuery(args));
		},
		
		createEditableTitleNode: function(titleNode, title, tooltip){
			if (this._editableTitle) {
				this._editableTitle.setValue(title);
				this.setDirty(false);
			} else {
				titleNode.innerHTML = jazz.util.html.escape(title);
				this._editableTitle = new dijit.InlineEditBox({
					title: tooltip, 
					autoSave: true, 
					width: "700px",
					noValueIndicator: this.getEditableTitleNodeDetails(), 
					editorParams: {maxLength: "60"},
					onFocus: dojo.hitch(this, this.hideMessage),
					onChange: dojo.hitch(this, this.onTitleChange)}, titleNode
				);
				dojo.addClass(this._editableTitle.domNode, "vvc-editor-editIndicator");
	
				this._editableTitle.startup();
			}
		},

		setTitleValue: function(value){
			if(this._editableTitle){
				this._editableTitle.set('value', value);
			}
		},
		
		onTitleChange: function(e){
			this.setDirty(true);
		},
		
		enableSaveButton: function()
		{
			var saveButton = dojo.byId("saveToolButton");
			if (saveButton) {
				saveButton.disabled = false;
			}
		},
		
		disableSaveButton: function()
		{
			var saveButton = dojo.byId("saveToolButton");
			if (saveButton) {
				saveButton.disabled = true;
			}
		},

		setDirty: function(flag) {
			var node = dojo.byId("dirty");
			if (flag === true) {
				// the focus event for the title may come after the mouse
				// click event save button, so we must use the disable
				// state before clearing existing error messages
				var saveButton = dojo.byId("saveToolButton");
				if (saveButton && saveButton.disabled != true) {
					this.hideMessage();
				} else if (this._createCompleted == true) {
					return;
				}

				this.enableSaveButton();
				if (!node.firstChild) {
					node.appendChild(dojo.doc.createTextNode(" *"));
				}
			} 
			else {
				this.disableSaveButton();
				if (node.firstChild) 
					node.removeChild(node.firstChild);
			}
		},

		getEditableTitleNodeDetails: function(){
			return "?";
		},
		
		_toolbarConstructs : {
			"createOperation" : [ "createToolButton", "cancelToolButton" ],
			"editOperationWithNoParent" : [ "saveToolButton", "cancelToolButton" ],	
			"editOperationWithParent" : [ "rebaseToolButton", "acceptToolButton", "deliverToolButton", "compareSnapshotToolButton", "compareParentToolButton", "saveToolButton", "cancelToolButton" ]
		},
		
		_initButtons: function() {
			if (this._isCreateOperation) {
				this._initToolbar("createOperation");
				var node = dojo.byId("parentPickerSpan");
				if (!node.firstChild && !this._parentUri) {
					var parentPickerButton = dojo.create("button");
					parentPickerButton.appendChild(dojo.doc.createTextNode(this.messages.selectParent));
					dojo.connect(parentPickerButton, "onclick",dojo.hitch(this, this._openPicker));
					node.appendChild(parentPickerButton);
				}
			}
			else
			{
				if (this._parentUri == null) 
					this._initToolbar("editOperationWithNoParent");
				else
					this._initToolbar("editOperationWithParent");
			}
		},
		
		_initToolbar: function(operation)
		{
			var operationsButtons = this._toolbarConstructs[operation];
			var toolbar = dojo.byId("workspace_editor_toolbar");
			var toolbarButtons = dojo.query(".jazz-ui-toolbar-Button, button", toolbar);
			if (operationsButtons != null)
			{
				toolbarButtons.forEach(function (button)
				{
					var id = button.id;
					if (dojo.indexOf(operationsButtons, id) >= 0)
						button.style.display = "";
					else
						button.style.display = "none";
				});
			}
		},
		
		_openPicker:function(e) {
			var self = this;
			var picker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog({
				"newButtonDisabled" : true,
			});
			
			picker.onOk = function(configuration) {
				self._clearFields();
				self._parentUri = configuration.uri;
				var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
				self._parentConfig = config;
				var link = PageUtil.getVvcUri() + "/repository/configuration?uri=" + self._parentUri;
				config.load(link, dojo.hitch(self, self.onLoadError));
				self.setSpaceField(config.getSpaceName());
				
				if (configuration.type == "workspace") {
					self.setParentWorkspaceField(configuration.title, configuration.uri);
					self.createParentSnapshotChoices();
				} else {
					self.setParentSnapshotField(configuration.title, configuration.uri);
					self.setParentWorkspaceField(config.getParentWorkspaceName(), config.getParentWorkspaceUri());
					self._snapshotComboBox = null;
				}
			};
		},
		
		getContextQuery: function() {
			if (this._workspace) {
				var uri = this._workspace.getUri();
				if (uri) {
					var query = {};
					query.contextUri= uri;
					query.breadcrumb = this._getEncodedWorkspaceTitle();
					return dojo.objectToQuery(query);
				}
			}
		},
		
		onCreateSnapshot: function(e) {
			var query = "operation=create";
			var context = this.getContextQuery();
			if (context) {
				query += "&" + context;
			}
			PageUtil.showPage(PageUtil.SNAPSHOT_PAGE, query);
		},

		onRebase: function() {
			var query = this.getContextQuery();
			PageUtil.showPage(PageUtil.REBASE_PAGE, query);
		},
		
		onDeliver: function() {
			var args = {};
			args.operation = "deliver";
			if (this._workspace) {
				args.descendantUri = this._workspace.getUri();
				args.ancestorUri = this._workspace.getParentSnapshotUri();
				args.otherDescendantUri = this._workspace.getParentWorkspaceUri();
				args.mergeUri = args.otherDescendantUri;
				args.breadcrumb = this._getEncodedWorkspaceTitle();
				
				// launch merge viewer
				PageUtil.showPage(PageUtil.COMPARE_VIEWER_PAGE, dojo.objectToQuery(args));
			}
		},
		
		onAccept: function() {
			var args = {};
			args.operation = "accept";
			if (this._workspace) {
				args.descendantUri = this._workspace.getUri();
				args.ancestorUri= this._workspace.getParentSnapshotUri();
				args.otherDescendantUri= this._workspace.getParentWorkspaceUri();
				args.mergeUri = args.descendantUri;
				args.breadcrumb = this._getEncodedWorkspaceTitle();
				
				// launch merge viewer
				PageUtil.showPage(PageUtil.COMPARE_VIEWER_PAGE, dojo.objectToQuery(args));
			}
		},
		
		onCompareWithParentSnapshot: function() {
			var args = {};
			args.operation = "compareWithParentSnapshot";
			if (this._workspace) {
				args.descendantUri = this._workspace.getUri();
				args.ancestorUri= this._workspace.getParentSnapshotUri();
				args.breadcrumb = this._getEncodedWorkspaceTitle();
				
				// launch merge viewer
				PageUtil.showPage(PageUtil.COMPARE_VIEWER_PAGE, dojo.objectToQuery(args));
			}
		},
		
		onCompareWithParentWorkspace: function() {
			var args = {};
			args.operation = "compareWithParentWorkspace";
			if (this._workspace) {
				args.descendantUri = this._workspace.getUri();
				args.ancestorUri= this._workspace.getParentWorkspaceUri();
				args.breadcrumb = this._getEncodedWorkspaceTitle();
				
				// launch merge viewer
				PageUtil.showPage(PageUtil.COMPARE_VIEWER_PAGE, dojo.objectToQuery(args));
			}
		},
		
		_getEncodedWorkspaceTitle: function() {
			return encodeURIComponent(this._workspace.getTitle());
		},
		
		refresh: function() {
			this._parentUri = this._workspace.getParentSnapshotUri();
			if (!this._parentUri) {
				// this workspace has no parent snapshot 
				this._parentUri = this._workspace.getParentWorkspaceUri();
			}
			this.setParentWorkspaceField(this._workspace.getParentWorkspaceName(), this._workspace.getParentWorkspaceUri());
			this.setParentSnapshotField(this._workspace.getParentSnapshotName(), this._workspace.getParentSnapshotUri());
			this.setSpaceField(this._workspace.getSpaceName());
			
			// create and set title
			var title = this._workspace.getTitle();
			this.createEditableTitleNode(this._titleNode, title, this.messages.clickToEdit);
			
			// set description text
			if (this._workspace.getDescription()) {
				this._descriptionField.value = this._workspace.getDescription();
			}
			this.connect(this._descriptionField, "onchange", function () {
				this.setDirty(true);
			});
			
			// build dimension viewer
			this._buildDimensionTab(this._workspace.getDimensions());
			
			// build dependency viewer
			var dependenciesContainer = dojo.byId("dependencies_viewer_div");
			if (this._dependenciesViewer) {
				// refresh this view
				var node = this._dependenciesViewer.domNode;
				dependenciesContainer.removeChild(node);
			}
			
			var params = { style: 2, lazyStart: true };
			if (this._workspace) {
				params.workspaceUri = this._workspace.uri;
			}
			this._dependenciesViewer = new com.ibm.team.vvc.server.core.ui.snapshots.WorkspacesAndSnapshotsViewer(params);
			dependenciesContainer.appendChild(this._dependenciesViewer.domNode);
			
			// build snapshots viewer
			var container = dojo.byId("snapshots_viewer_div");
			if (this._snapshotsViewer) {
				container.removeChild(this._snapshotsViewer.domNode);
			}
			
			var args = { workspace: this._workspace, style: 0, lazyStart: true };
			this._snapshotsViewer = new com.ibm.team.vvc.server.core.ui.snapshots.SnapshotsViewer(args);
		    container.appendChild(this._snapshotsViewer.domNode);

		    // build workspaces viewer
			var container = dojo.byId("workspaces_viewer_div");
			if (this._workspacesViewer) {
				container.removeChild(this._workspacesViewer.domNode);
			}
			
			var args = { workspace: this._workspace, style: 2, lazyStart: true };
			this._workspacesViewer = new com.ibm.team.vvc.server.core.ui.snapshots.SnapshotsViewer(args);
		    container.appendChild(this._workspacesViewer.domNode);
    		
			// build changesets viewer
			var container = dojo.byId("changesets_viewer_div");
			if (this._changesetsViewer) {
				container.removeChild(this._changesetsViewer.domNode);
			}
			
			var args = { workspace: this._workspace, style: 3, lazyStart: true };
			this._changesetsViewer = new com.ibm.team.vvc.server.core.ui.snapshots.SnapshotsViewer(args);
		    container.appendChild(this._changesetsViewer.domNode);
		    
			// build resource mappings viewer
			var container = dojo.byId("resources_viewer_div");
			if (this._resourceMappingTable) {
				// refresh this view
				container.removeChild(this._resourceMappingTable.domNode);
			}
			
			var args = { configurationUri: this._workspace.uri, lazyStart: true };
			this._resourceMappingTable = new com.ibm.team.vvc.server.core.ui.internal.page.ResourceMappingViewer(args);
		    container.appendChild(this._resourceMappingTable.domNode);
		    
		    this._tabManager.showActiveTab(this._tabOverview);
		},
		
		createParentSnapshotChoices: function() {
			if (this._parentContext.firstChild) {
				this._parentContext.removeChild(this._parentContext.firstChild);
			}
			var select = dojo.create("select", null, this._parentContext);
			this._snapshotComboBox = select;

			// add create snapshot automatically option
			var option = dojo.create("option", {
				value: "autoCreateBaseSnapshot",
				selected: true
			});
			option.appendChild(dojo.doc.createTextNode(this.messages.autoCreate));
			select.appendChild(option);
			option.title = this.messages.autoCreateTooltip;
			
			var uri = PageUtil.getVvcUri() + "/repository/snapshots?parentWorkspace=" + escape(this._parentUri);
			var args = {
				url: uri,
				handleAs: "json",
				headers: {
					Accept: "application/json;application/rdf+json;text/rdf+json;text/json"
				},
			};
			var deferred = jazz.client.xhrGet(args);
			
			// create options for the select box
			deferred.addCallback(this, function (snapshots) {
				for (var i = 0; i < snapshots.length; i++){
					var snapshotEntry = snapshots[i];
					for (var snapshotUri in snapshotEntry) {
						var snapshot = snapshotEntry[snapshotUri];
						option = dojo.create("option", {
							value: snapshotUri
						});

						// create snapshot name entry
						var titleNode = snapshot[Constants.TITLE];
						if (titleNode && titleNode[0]) {
							var name = titleNode[0].value;
							if (name) {
								option.appendChild(dojo.doc.createTextNode(name));
								select.appendChild(option);
							}
						}

						// create tooltip
						var createdNode = snapshot[Constants.CREATED];
						if (createdNode && createdNode[0]) {
							var created = createdNode[0].value;
							if (created) {
								var createdDate = dojo.date.stamp.fromISOString(created);
								var date = dojo.date.locale.format(createdDate, {formatLength: "medium"});
								var text = dojo.string.substitute(this.messages.createdDate, [date]);
								
								// append description if not empty
								var descriptionNode = snapshot[Constants.DESCRIPTION];
								if (descriptionNode && descriptionNode[0]) {
									var desc = descriptionNode[0].value;
									if (desc) {
										text += " \n" + this.messages.description + " " + desc;
									}
								}
								option.title = text;
							}
						}
					}
				}
				
				// add none option for no snapshot in last entry
				option = dojo.create("option", {
					value: NON_BASE_SNAPSHOT
				});
				option.appendChild(dojo.doc.createTextNode(this.messages.none));
				select.appendChild(option);
				option.title = this.messages.withoutBaseSnapshotTooltip;
			});
			
			deferred.addErrback(this, function(error) {
				this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
				this.enableActions();
				
								
				// add none option for no snapshot in last entry
				option = dojo.create("option", {
					value: NON_BASE_SNAPSHOT
				});
				option.appendChild(dojo.doc.createTextNode(this.messages.none));
				select.appendChild(option);
				option.title = this.messages.withoutBaseSnapshotTooltip;
			});
		},
		
		setParentSnapshotField: function(parentSnapshotName, uri) {
			if (!parentSnapshotName || parentSnapshotName == null) {
				parentSnapshotName = messages.none;
			}
			PageUtil.setLinkField(this._parentContext, parentSnapshotName, uri, PageUtil.SNAPSHOT_PAGE);
		},
		
		setParentWorkspaceField: function(parentWorkspaceName, uri, pageId) {
			if (!parentWorkspaceName || parentWorkspaceName == null) {
				parentWorkspaceName = messages.none;
			}
			PageUtil.setLinkField(this._parentWorkspace, parentWorkspaceName, uri, PageUtil.WORKSPACE_PAGE);
		},
		
		setSpaceField: function(spaceName) {
			PageUtil.setLinkField(this._configurationSpace, spaceName);
		},
		
		_buildDimensionTab: function(dimensions) {
			// create table
			var columnData = [{ label: this.messages.name }, { label: this.messages.value }];
			var container = this._dimensionContainerNode;
			if (container && container.firstChild) {
				container.removeChild(container.firstChild);
			}
			
			// create table header
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: columnData
			});
			table.domNode.style.width = "100%";
			container.appendChild(table.domNode);
			
			// Now add the rows
			if (dimensions && dimensions.length > 0) {
				var tr, td;
				for (var i = 0; i < dimensions.length; i++){
					tr = document.createElement("tr");
					var nameValuePair = dimensions[i].value;
					var index = nameValuePair.indexOf("\n");
					if (index != -1) {
						var name = nameValuePair.substring(0, index);
						var value = nameValuePair.substring(index + 1);
						var link = null;
						var pageId = null;
						
						if ("TimeOrder" === name || messages.timeOrderDimension === name) {
							name = messages.timeOrderDimension;
							pageId = PageUtil.SNAPSHOT_PAGE;
							if (this._isCreateOperation && this._parentConfig) {
								link = this._parentConfig.getUri();
							} else if (this._workspace) { // editing existing workspace
								link = this._workspace.getParentSnapshotUri();
							}
						} else if ("One Greater Than" === name || messages.oneGreaterThanDimension == name) {
							name = messages.oneGreaterThanDimension;
							pageId = PageUtil.WORKSPACE_PAGE;
							if (this._isCreateOperation && this._parentConfig) {
								link = this._parentConfig.getParentWorkspaceUri();
							} else if (this._workspace) { // editing existing workspace
								link = this._workspace.getParentWorkspaceUri();
							}
						}
						
						// name column
						td = document.createElement("td");
						td.innerHTML = name;
						tr.appendChild(td);
						
						// value column
						td = document.createElement("td");
						if (link != null && pageId != null) {
							PageUtil.setLinkField(td, value, link, pageId);
						} else {
							td.innerHTML = value;
						}
						tr.appendChild(td);
					}
					
					table.tbody.appendChild(tr);
				}
			}
			
			return table;
		},
		
		onSave: function() {
			this.hideMessage();
			var title = this._editableTitle.value;
			var description = this._descriptionField.value;
			if (!title || title.length === 0) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.emptyName
				});
				return;
			}
			
			var deferred;
			this.disableActions();
			if (this._isCreateOperation) {
				UIUtils.showPageMessage(messages.creatingNewWorkspace);
				if (this._spaceUri == null && this._parentConfig != null) {
					this._spaceUri = this._parentConfig.getSpaceUri();
				}
				var configUri = PageUtil.findConfigurationName(title, this._spaceUri, true);
				if (configUri != null) {
					var config = this._getConfiguration(configUri);
					if (config != null && config.isMutable()) {
						this.enableActions();
						return this._showDuplicateNameError();
					}
				}
				
				if (this._parentConfig && this._parentConfig.isMutable() && this._snapshotComboBox) {
					// handle create workspace with parent workspace context
					var index = this._snapshotComboBox.selectedIndex;
					if (index == 0) {
						// create new snapshot and new workspace
						var baseSnapshotName = dojo.string.substitute(this.messages.autoCreateSnapshotLabel, [title]);
						deferred = PageUtil.createSnapshot(baseSnapshotName, "",  this._parentConfig.getUri(), this._spaceUri);
						deferred.addCallback(this, function(params){
							var baseSnapshotUri = params;
							this._createWorkspace(title, description, baseSnapshotUri);
						});
						deferred.addErrback(this, function(error) {
							var xhrError = PageUtil.createXhrError(deferred.ioArgs);
							this.showMessage({
								type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
								summary: xhrError.message,
								details: xhrError.detailedMessage
							});
							this.enableActions();
						});
					} else {
						// create new workspace with selected existing snapshot
						var snapshot = this._snapshotComboBox.options[index];
						if (snapshot.value === NON_BASE_SNAPSHOT) {
							this._createWorkspace(title, description, this._parentUri);
						} else {
							this._createWorkspace(title, description, snapshot.value);
						}
					}
				} else {
					this._createWorkspace(title, description, this._parentUri);
				}
				
			} else {
				UIUtils.showPageMessage(messages.saving);
				var saveButton = dojo.byId("saveToolButton");
				this._workspace.setTitle(title);
				if (this._workspace.isTitleChanged()) {
					var configUri = PageUtil.findConfigurationName(title, this._workspace.getSpaceUri(), true);
					if (configUri != null) {
						config = this._getConfiguration(configUri);
						if (config != null && config.isMutable() && config.uri !== this._workspace.uri) {
							this.enableActions();
							return this._showDuplicateNameError();
						}
					}
				}
				this._workspace.setDescription(description);
				deferred = this._workspace.save();
				deferred.addCallback(this, function(params){
					var returnLink = PageUtil.getReturnLink(this._workspace.uri);
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.OK,
						summary: messages.saveSuccessful+returnLink
					});
					this.enableActions();
					this.setDirty(false);
				});
				deferred.addErrback(this, function(error) {
					this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
					this.enableActions();
				});
			}
		},
		
		_createWorkspace: function(title, description, baseSnapshotUri) {
			deferred = PageUtil.createWorkspace(title, description, baseSnapshotUri, this._spaceUri);
			deferred.addCallback(this, function(params){
				var returnLink = PageUtil.getReturnLink(params);
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: messages.createSuccessful + returnLink
				});
				this.setDirty(false);
				this._createCompleted = true;
			});
			deferred.addErrback(this, function(error) {
				var xhrError = PageUtil.createXhrError(deferred.ioArgs);
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: xhrError.message,
					details: xhrError.detailedMessage
				});
				this.enableActions();
			});
		},
		
		onLoadError: function(response, ioArgs) {
			var xhrError = PageUtil.createXhrError(ioArgs);
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: xhrError.message,
				details: xhrError.detailedMessage
			});
		},
		
		enableActions: function(enableSave)
		{
			var createButton = dojo.byId("createToolButton");
			if (createButton) {
				createButton.disabled = false;
			}
			this.enableSaveButton();
			var cancelButton = dojo.byId("cancelToolButton");
			if (cancelButton) {
				cancelButton.disabled = false;
			}
			if (this._editableTitle)
				this._editableTitle.setDisabled(false);
			if (this._descriptionField)
				this._descriptionField.disabled = false;
		},
		
		disableActions: function()
		{
			var createButton = dojo.byId("createToolButton");
			if (createButton) {
				createButton.disabled = true;
			}
			var saveButton = dojo.byId("saveToolButton");
			if (saveButton) {
				saveButton.disabled = true;
			}
			var cancelButton = dojo.byId("cancelToolButton");
			if (cancelButton) {
				cancelButton.disabled = true;
			}
			this._editableTitle.setDisabled(true);
			this._descriptionField.disabled = true;
		},
	})   
})();
