/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

dojo.provide("com.ibm.team.vvc.server.core.ui.internal.wizard.FinalizeStep");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.wizard", "FinalizeStepMessages");

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;

(function() {
dojo.declare("com.ibm.team.vvc.server.core.ui.internal.wizard.FinalizeStep", com.ibm.team.repository.web.ui.internal.wizard.FinalizeStep, {
	
	userSyncCompleted: false,
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.wizard", "FinalizeStepMessages");
		this.stepTitle = messages.stepTitle;
	},

	postCreate: function() {
		this.inherited(arguments);
		this.nowSetCompletedState(false);

		// Create DOM for the individual finalize operations
		this.domApplyLicense = this._addOperationDiv(this.messages.operation3title, this.messages.operation3desc);
		this.domConfigureStorage = this._addOperationDiv(this.messages.operation4title, this.messages.operation4desc);
	},
	
	isFinalizationNeeded: function() {
		var self = this;
		var dFinalize = this.finalizationIsNeeded();
		dFinalize.addCallback(function(response){
			if(response === "true") {
				//Finalize needed
				var srh = new ServiceResponseHandler(self, "setJTSUrl", "handleAppNotRegistered");
				ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
			} else {
				self.header.showMessage({type: "OK", summary: self.messages.noFinalizeNeeded });
				self.nowSetCompletedState(true);
				dojo.addClass(self.progressSection, "noDisplay");
			}
		});
		dFinalize.addErrback(function(errObject){
			//Allow the user to try to finalize since we don't know if they have already done so.
			self._handleError(self.messages.finalizeCheckError, errObject);
			self.startButton.disabled = false;
		});
	},
	
	showCompletedMessage: function() {
		// overridden
		// userSync calls this when it is successful
		// Note: it may partially be successful but thats still ok...
		this.applyInternalLicense();
	},
	
	_setCompletedState: function(state) {
		// overridden
		// userSync calls this when it completes.
		// We dont want to do anything here.
	},
	
	nowSetCompletedState: function(state) {
		this.isCompleted = state;
		this.triggerButtonStateUpdate();
	},
	
	nowShowCompletedMessage: function() {
		var buttonLabel = this.useFinishButton() === true ? this.messages.finish : this.messages.next;
		this.header.showMessage({
			type: "OK",
			summary: dojo.string.substitute(this.messages.allComplete, [ buttonLabel ])
		});
		this.nowSetCompletedState(true);
	},
	
	applyInternalLicense: function() {
		var self = this;
		var domNode = this.domApplyLicense;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/setup-descriptor/applylicense',
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.createStorageAreas();
		});
		dFinalize.addErrback(function(errObject){
			self._showInlineError(domNode, self.messages.licenseError, errObject);
		});
	},
	
	createStorageAreas: function() {
		var self = this;
		var domNode = this.domConfigureStorage;
		var deferred = new dojo.Deferred();
		dojo.addClass(domNode, "loading");
		jazz.client.xhrPost({
			url: net.jazz.ajax._contextRoot + '/setup-descriptor/createstorage',
			load: function(response) {
				deferred.callback(response);
			},
			error: function(errObject) {
				deferred.errback(errObject);
			}
		});	
		deferred.addCallback(function(response){
			dojo.removeClass(domNode, "loading");
			dojo.addClass(domNode, "ok");
			self.nowSetCompletedState(true);
			self.nowShowCompletedMessage();
		});
		dFinalize.addErrback(function(errObject){
			//Allow the user to try to finalize since we don't know if they have already done so.
			self._showInlineError(domNode, self.messages.createStorageError, errObject);
		});
	},
	
	_showInlineError: function(node, message, errObject) {
		this.inherited(arguments);
		this.nowSetCompletedState(false);
	},	
	
	_handleError: function(message, response) {
		this.inherited(arguments);
		this.nowSetCompletedState(false);
	},
});

})();
