/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog');

dojo.require('jazz.app.auth');
dojo.require('jazz.ui.Dialog');
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.MessageBox");
dojo.require('jazz.ui.SearchBox');
dojo.require("jazz.ui.StyledTable");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");

dojo.require('dojo.NodeList-traverse');
dojo.require('dojo.i18n');
dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.picker", "WorkspacePickerDialogMessages");

(function ()
{
	
	function ignoreFieldKeyEvent(e)
	{
		if (e.ctrlKey || e.altKey) return true;

		switch(e.keyCode)
		{
		case dojo.keys.ENTER:
		case dojo.keys.ESCAPE:
		case dojo.keys.LEFT_ARROW:
		case dojo.keys.RIGHT_ARROW:
		case dojo.keys.TAB:
		case dojo.keys.SHIFT:
		case dojo.keys.ALT:
		case dojo.keys.CAPS_LOCK:
		case dojo.keys.PAGE_DOWN:
		case dojo.keys.PAGE_UP:
		case dojo.keys.LEFT_WINDOW:
		case dojo.keys.RIGHT_WINDOW:
		case dojo.keys.HOME:
		case dojo.keys.END:
		case dojo.keys.CTRL:
		case dojo.keys.F5:
			{
			return true;
			}
			
		default:
			return false;
		}
	}
	
	function escapeHtml(html) {
		return html.replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;');
	}
	
	/**
	 * Set "spaceUri" property to set the space uri you would like this dialog to constrain
	 * it's search results to, in the first argument of the constructor.
	 */
	dojo.declare(
		'com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog',
		[dijit._Widget, dijit._Templated], 
		{
			widgetsInTemplate: true,
			templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/picker/templates/WorkspacePickerDialog.html'),
			
			onCancel: null,

			onOk: null,

			dialogTitle: null, // defaults to 'Select a Resource'
			
			constructor: function (args)
			{
				if (args)
				{
					if (args.spaceUri)
						this._spaceUri = args.spaceUri;
				}
				this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.picker", "WorkspacePickerDialogMessages");
				this.dialogTitle = this.messages.dialogTitle;
				this.spinnerUrl = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/spinner.gif').uri;
				this._iconSrc = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/VVC.gif').uri;
			},

			cancel: function() {
				this._dialog.close();
				this.onCancel && this.onCancel();
				this.destroyRecursive();
			},

			postCreate: function() {
				var self = this;
				
				this._detailsOn = false;
				this._createDetailsButton();
				this._createResultsTable();
				this._createDimensionsTable();
				this._dialog.set('onClose', function() {
					self.onCancel && self.onCancel();
					self.destroyRecursive();
				});
				
				// set initial focus
				if (this._searchBox) {
					this._searchBox.focus();
				}
			},
			
			_createDetailsButton: function()
			{
				var self = this;
				this.toolbar = new jazz.ui.SimpleToolbar({});
				this.detailsToggleButton = this.toolbar.addButton(this, function() {self._toggleDetails();}, {
					//TODO: find good hide details icon
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/show_details.gif'),
					label: 'Toggle Details'
				});
				this._resultsTableToolbar.appendChild(this.toolbar.domNode);
			},
			_toggleDetails: function()
			{
				if (this._detailsOn)
				{
					this._detailsOn = false;
					this._dimensionsTableSection.style.display = "none";
					this._dialog.setWidth('411px');
					//TODO: find some good icon for details hiding and showing
					this.toolbar.updateButton(this.detailsToggleButton, { imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/show_details.gif'), label: 'Toggle Details'});
				}
				else
				{
					this._detailsOn = true;
					this._dimensionsTableSection.style.display = "";
					this._dialog.setWidth('664px');
					this.toolbar.updateButton(this.detailsToggleButton, { imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/show_details.gif'), label: 'Toggle Details'});
				}
			},
			_createResultsTable: function()
			{
				var table = new jazz.ui.StyledTable({
					   columns: [
					      {label: "", width: "16px", sortable: false},
					      {label: "Name", width: "40%", sortable: false},
					      {label: "Description", sortable: false}
					   ],
					   sortBy: 1,
					   sortByAscending: false,
					   handleSort: function() {
					      alert('do sort');
					   },
				});
				
				table.domNode.style.width = "100%";
				this._resultsTable = table;
				this._resultsTableDiv.appendChild(table.domNode);
			},
			
			_createDimensionsTable: function()
			{
				var table = new jazz.ui.StyledTable({
					   columns: [
					      {label: "Dimension Name", sortable: false},
					      {label: "Dimension Value", sortable: false}
					   ],
				});
				
				table.domNode.style.width = "100%";
				this._dimensionsTable = table;
				this._dimensionsTableDiv.appendChild(table.domNode);
			},

			startup: function() {
				this.inherited(arguments);
				var self = this;
				window.setTimeout(function() {
					self._searchBox.focus();
				}, 1);
			},

			getSelected: function()
			{
				return this._selectedRow ? this._selectedRow.workspace : undefined;
			},
			
			_selectRow: function(row) {
				if (this._selectedRow) dojo.removeClass(this._selectedRow, 'selected');
				
				if (!row.workspace)
					return;
				
				dojo.addClass(row, 'selected');
				this._selectedRow = row;

				this._submitButton.disabled = false;
				
				this._loadDimensionValuesIntoTable(row.workspace.dimensionValueSet);

			},
			
			_loadDimensionValuesIntoTable: function(dimensionValueSet)
			{
				var frag = document.createDocumentFragment();
				
				this._dimensionsTable.clearBody();

				var tr, td;
				for (var i = 0; i < dimensionValueSet.length; i++) {
					var dimensionValue = dimensionValueSet[i];
					
					tr = document.createElement("tr");
					
					td = document.createElement("td");
					td.innerHTML = dimensionValue.dimensionName;
					tr.appendChild(td);
					
					td = document.createElement("td");
					td.innerHTML = dimensionValue.dimensionValue;
					tr.appendChild(td);
					frag.appendChild(tr);
				}
				
				this._dimensionsTable.tbody.appendChild(frag);
			},

			_handleArrow: function(key) {
				if (!this._lastResult) return;

				if (key == dojo.keys.DOWN_ARROW)
				{	
					if (!this._selectedRow) {
						// select first on arrow down if no selection
						this._selectRow(this._getFirstRow(this._resultsTable));
					}
					else if (this._selectedRow.nextSibling && this._selectedRow.nextSibling.nodeName == 'TR') {
						this._selectRow(this._selectedRow.nextSibling);
					}
				}
				else if (key == dojo.keys.UP_ARROW)
				{
					if (!this._selectedRow) {
						// select last on arrow up if no selection
						this._selectRow(this._getLastRow(this._resultsTable));
					}
					else if (this._selectedRow.previousSibling && this._selectedRow.previousSibling.nodeName == 'TR') {
						this._selectRow(this._selectedRow.previousSibling);
					}
				}
			},

			_onFieldKeyDown: function(e) {
				if (ignoreFieldKeyEvent(e)) return;
				e.stopPropagation();
				var k = e.keyCode;
				
				if (k == dojo.keys.UP_ARROW
					|| k ==  dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this._handleArrow(k);
					return;
				}

				this._deselectAll();
				window.clearTimeout(this._pendingSearch);
				this._pendingSearch = window.setTimeout(dojo.hitch(this, this._runSearch, null), 400);
			},
			
			_onListKeyDown: function(e) {
				var k = e.keyCode;
				
				if (k == dojo.keys.UP_ARROW
					|| k ==  dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this._handleArrow(k);
				}
			}, 
			
			_onDblClickResource: function(e) {
				var row = e.target;

				if (e.target.nodeName != 'TR') {
					row = dojo.NodeList(e.target).closest('TR');
					if (!row) {
						console.warn('no list item found on click');
						return;
					}
					row = row[0];
				}

				dojo.stopEvent(e);
				this._selectRow(row);
				this._onSubmit();
			},

			_onClickResource: function(e) {
				var row = e.target;

				if (e.target.nodeName != 'TR') {
					row = dojo.NodeList(e.target).closest('TR');
					if (!row) {
						console.warn('no list item found on click');
						return;
					}
					row = row[0];
				}

				if (dojo.hasClass(row, 'selected')) {
					return;
				}
				else {
					this._deselectAll();
					this._selectRow(row);
				}
			},
			
			_deselectAll: function()
			{
				if (this._selectedRow) {
					dojo.removeClass(this._selectedRow, 'selected');
					delete this._selectedRow;
				}

				this._submitButton.disabled = true;
			},
			
			_showLoading: function(show)
			{
				if (show) {
					this._loadingNode.style.display = '';
					this._noResultsNode.style.display = 'none';
				}
				else {
					this._loadingNode.style.display = 'none';
				}
			},

			_updateCount: function(count) {
				if (count == 0) {
					this._loadingNode.style.display = 'none';
					this._noResultsNode.style.display = '';
					this._noResultsNode.innerHTML = escapeHtml(this.messages.noResults);
				}
				else {
					this._noResultsNode.style.display = 'none';
				}
			},
			
			uninitialize: function()
			{
				this.inherited(arguments);
			},

			_runSearch: function(e) {
				e && dojo.stopEvent(e);

				if (e && e.target == this._searchIcon) {
					this._searchBox.select();
				}

				window.clearTimeout(this._pendingSearch);

				var searchString = dojo.trim(this._searchBox.value);
				
				this.hideErrorMessage();
				
				delete this._lastResult;
				this._submitButton.disabled = true;
				this._showLoading(true);
				this._resultsTable.clearBody();
				this._dimensionsTable.clearBody();
				
				if (searchString.length == 0) {
					this._updateCount(0, 0);
				}

				var self = this;
				
				var content = {
					query: encodeURI(searchString)
				};

				if (this.spaceUri)
					content["space"] = encodeURI(this._spaceUri);
				
				jazz.client.xhrGet({
					handleAs: 'json',
					url: net.jazz.ajax.contextRoot() + '/vvcshared/proxy', //$NON-NLS-1$
					"content": content,
					"headers" : {
						"vvc_servicePath" : "/WorkspacesQuery/query"
					},	
					handle: dojo.hitch(this, function(response, ioargs)
					{
						this._handleResponse(response, ioargs);
					}),
				});
			},
			
			_handleResponse: function(response, ioargs) {
				this._showLoading(false);
				this._deselectAll();
				dojo.removeClass(this._searchIcon, 'loading');
				
				if (ioargs.xhr.status == 200)
				{
					if ("errorMessage" in response) //TODO: This is to be modified to whatever the error is supposed to be in the error json format
						this.displayErrorMessage(response.errorMessage);
					else  
						this._handleLoad(response);
				}
				else {
					console.log('error', response);
					this.displayErrorMessage("Invalid Response (Status:"+ioargs.xhr.status+"): "+ response.responseText);
				}
			},
			
			displayErrorMessage: function(errorMessage) 
			{
				var mb = new jazz.ui.MessageBox();
				mb.placeAt(dojo.byId("errorMessages"));
				mb.showMessage({summary: errorMessage, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR});
				dojo.style("errorMessages", {display: ""});
			},
			
			hideErrorMessage: function() {
				dojo.empty("errorMessages");
				dojo.style("errorMessages", {display: "none"});
			},
			
			_handleLoad: function(result)
			{
				var frag = document.createDocumentFragment();
				var workspaces = result.items;

				var missingIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/missing_icon.gif');
				
				var tr, td;
				for (var i = 0; i < workspaces.length; i++) {
					var workspace = workspaces[i];
					
					tr = document.createElement("tr");
					
					td = document.createElement("td");
					td.appendChild(dojo.create('img', {
						src: this._getImgURL(workspace),
						height: '16',
						width: '16'
						}));
					tr.appendChild(td);
					
					td = document.createElement("td");
					var titleNode = document.createTextNode(workspace.title);
					td.appendChild(titleNode);
					tr.appendChild(td);
					
					td = document.createElement("td");
					td.innerHTML = workspace.description;
					tr.appendChild(td);
				
					tr.workspace = workspace;
					frag.appendChild(tr);
				}

				this._lastResult = result;
				this._resultsTable.clearBody();
				
				this._resultsTable.tbody.appendChild(frag);
				this._updateCount(workspaces.length);
			},
			
			_getImgURL: function(workspace) {
				if (workspace.type == 'snapshot')
					return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/snapshot.gif').uri;
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/workspace.gif').uri;
			},
			
			_onSubmit: function(e) {
				e && dojo.stopEvent(e);

				if (this._selectedRow) {
					// There are selected result, so OK the dialog.
					// Can get selected result still with getter until this is destroyed.

					// Prevent the onClose callback since we already know it's happening in this case
					delete this._dialog.onClose;
					this._dialog.close();

					if (this.onOk) {
						// get off the DOM before destroy
						var workspace = this._selectedRow.workspace;
						var self = this;

						// Delay the callback until after the 'destroy' so that the dialog goes away
						window.setTimeout(function() {
							self.onOk(workspace);
						}, 1);
						
					}
					this.destroyRecursive();
				}
			},
			
			_getFirstRow: function(table) {
				var children = table.domNode.childNodes;
				for (var i = 0; i < children.length; i++) {
					var child = children[i];
					if (child.nodeName == 'TR')
						return child;
				}
				return null;
			},
			
			_getLastRow: function(table) {
				var children = table.domNode.childNodes;
				for (var i = children.length - 1; i > -1; i--) {
					var child = children[i];
					if (child.nodeName == 'TR')
						return child;
				}
				return null;
			},
			
		}
	);
})();
