/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.workspaces.WorkspacesTreeGrid");

dojo.require("dijit._Widget");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("dojox.grid.LazyTreeGrid");
dojo.require("dojox.grid.LazyTreeGridStoreModel");
dojo.require("dojo.data.ItemFileWriteStore");

dojo.require('dojo.i18n');
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.PageTemplate");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
dojo.require("com.ibm.team.vvc.server.core.ui.workspaces.ConfigurationsLazyTreeGrid");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.workspaces", "WorkspacesPageMessages");

(function ()
{
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.workspaces", "WorkspacesPageMessages");
	
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	
	var contextRoot = net.jazz.ajax._contextRoot;
	if (contextRoot.indexOf('jazz') != -1) contextRoot += '/jfs';
	
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.workspaces.WorkspacesTreeGrid", [dijit._Widget], {
		
		constructor: function (args)
		{
			if (args)
			{
				if (args.displayErrorMessage)
					this.displayErrorMessage = args.displayErrorMessage;
				if (args.hideErrorMessage)
					this.hideErrorMessage = args.hideErrorMessage;
			}
		},
		
		postCreate: function(){
			var self = this;
			
			this._constructGrid();
		},
	   		
		_constructGrid: function() {
			if (!this._grid)
			{
				var model = this._createModel([]);
		
			    /* set up layout */
			    var layout = [
			                  {name: 'Actions', field: null, width: '75px', formatter: dojo.hitch(this, this.formatActions)},
			                  {name: 'Name', field: 'title', width: 'auto', formatter: dojo.hitch(this, this.formatName)},
			                  {name: 'Space', field: 'spaceTitle', width: '300px', formatter: dojo.hitch(this, this.formatSpaceTitle)},
			    ];
		
				this._loadData(
						dojo.hitch(this, function (data) {
							this._grid = new com.ibm.team.vvc.server.core.ui.workspaces.ConfigurationsLazyTreeGrid({
						        id: 'grid',
						        treeModel: this._createModel(data),
						        structure: layout,
						        rowSelector: '0px',
						        expandoCell: 1,
						        loadingMessage: messages.loading,
						        autoHeight: true,
						      }, document.createElement('div'));
						    
						    this.domNode.appendChild(this._grid.domNode);
						    this._grid.startup();
						}),
						dojo.hitch(this, function (e) {
							console.log(e);
							if (this.displayErrorMessage)
								this.displayErrorMessage("Error: "+e.message);
							else
								alert('error '+e.message);
						})
					);
				
			}
		},
		
		escapeHtml: function(str){
			return str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;")
			.replace(/"/gm, "&quot;").replace(/'/gm, "&#39;");
		},
		
		formatName: function(value, inRowIndex, level, cell)
		{
			var model =  cell.grid.treeModel;
			var item = cell.grid.getItem(inRowIndex);
			var iconURL = this.getImageUrl(item, model, cell.grid);
			
			var label = "";
			if (value) {
				label = this.escapeHtml(value);
			}
			var openEditorScript = "com.ibm.team.vvc.server.core.ui.internal.util.PageUtil.openEditor(\'"+item.type+"\', \'"+item.uri+"\', \'"+item.spaceUri+"\')";
			return "<a onclick=\"javascript: "+openEditorScript+";\" style=\"cursor: pointer;\" ><img src="+"\""+iconURL+"\"" + " style=\"padding-top: 2px; padding-right: 4px;\"  height=\"16px\" width=\"16px\"\>" + label+"</a>";
		},
		
		formatSpaceTitle: function(value, inRowIndex, level, cell)
		{			
			var label = "";
			if (value) {
				label = this.escapeHtml(value);
			}
			return "<span style=\"color:#666666; margin-left:4px;\" \>" + label+"</span>";
		},
		
		getImageUrl: function(item, model, grid){
			if (item.type == "workspace")
				return dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/workspace.gif').uri;
			else if (item.type == "snapshot")
				return dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/snapshot.gif').uri;
			return "";
		},
		
		_actionRegistry: [
			{
				isApplicable: function(node) {
					return !node._RI;
				},
				label:  messages.compare,
				imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/compare-enabled.gif').uri,
				handler: function (node) {
					var query = {};
					query.ancestorUri = "previous"; // keyword for compare service to figure out the actual uri
					query.descendantUri = node.uri;
					PageUtil.showAction(PageUtil.COMPARE_VIEWER_PAGE, dojo.objectToQuery(query));
				}
			},
			/*{
				isApplicable: function(node) {
					return !node.children; 
				},
				label:  messages.remove,
				imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/delete.gif').uri,
				handler: function (node) {
					alert('TODO: delete');
				}
			},*/              
			{
				isApplicable: function(node) {
					return true;
				},
				label:  messages.newWorkspace,
				imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/create_workspace-enabled.gif').uri,
				handler: function (node) {
					var query = "contextUri=" + escape(node.uri) + "&spaceUri="+escape(node.spaceUri);
					PageUtil.showAction("vvc.createWorkspace", query);
				}
			},
			{
				isApplicable: function(node) {
					return node.mutable == 'true'; 
				},
				label:  messages.newSnapshot,
				imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/create_snapshot-enabled.gif').uri,
				handler: function (node) {
					var query = "contextUri=" + escape(node.uri) + "&spaceUri="+escape(node.spaceUri);
					PageUtil.showAction("vvc.createSnapshot", query);
				}
			},
		],

		
		formatActions: function(value, inRowIndex, level, cell) 
		{
			var model =  cell.grid.treeModel;
			var toolbar = new jazz.ui.Toolbar();
			this._grid.toolbars[inRowIndex] = toolbar;
			var node = cell.grid.getItem(inRowIndex);
			
			dojo.forEach(this._actionRegistry, dojo.hitch(this, function (action)
			{
				var button = jazz.ui.toolbar.Button({
					title: action.label,
					imgSrc: action.imgSrc,
					onClick: dojo.hitch(this, function(){
						action.handler.call(this, node);
					})
				});
				if (!action.isApplicable(node))
					button.set("enabled", false);
				toolbar.addChild(button);
			}));
			dojo.addClass(toolbar.domNode, "com-ibm-team-vvc-workspaces-explorerActionsToolbar");		
			return toolbar;
		},
		
		resize: function()
		{
			if (this.grid) { this._grid.resize(); }
		},
		
	    uninitialize: function()
		{
	    	if (this._grid) {
				this._grid.destroyRecursive();
				delete this._grid;
			}
		},
		
		collapseAll:function()
		{
			for (var i = this._grid.cache.items.length -1; i >= 0; i--) {
				if (this._grid.cache.getExpandoStatusByRowIndex(i)){
					this.toggleExpandRow(i);
				}
			}
		},
		
		getView: function() {
			return this._grid.views.views[1];
		},
		
		toggleExpandRow: function(ind) {
			var idty = this._grid._by_idx[ind].idty; 
			var expando = this.getView()._expandos[idty];
			if (expando) {
				expando.onToggle({stopPropagation: function() {}, preventDefault: function() {}} , true);
			}
		},
		
		_loadData: function (onResult, onError) {
			jazz.client.xhrGet({
				handleAs: 'json',
				url: contextRoot + '/WorkspacesExplorer/query', //$NON-NLS-1$
				sync: false,
				content: {},
				load: onResult, 
				error: onError
			});
		},
		
		_createModel: function (data) {
			var store = new dojo.data.ItemFileWriteStore({data: this.data = data});
		    return new dojox.grid.LazyTreeGridStoreModel({store: store, childrenAttrs: ['children']});
		},
		
		refresh: function()
		{
			UIUtils.showPageMessage("Refreshing explorer");
			this._loadData(
					dojo.hitch(this, function (data) {
						if (data) {
							this._grid.showMessage();
							this._grid.setModel(this._createModel(data));
						} else {
							this._grid.showMessage(messages.noConfigurationsDefined);
						}
					}),
					dojo.hitch(this, function (e) {
						console.log(e);
						if (this.displayErrorMessage)
							this.displayErrorMessage("Error: "+e.message);
						else
							alert('error '+e.message);
					})
				);
			UIUtils.hidePageMessage();
		},
		
	    
	    _compareSelected: function()
	    {
	    	if (!this._grid || !this._grid.selection) {
	    		return;
	    	}
	    	var selected = this._grid.selection.selected;
	    	var items = [];
	    	for (var i in selected) {
	    		if (selected[i]) {
	    			var element = this._grid.cache.items[i];
	    			items.push(element.item.uri);
	    		}
	    	}

	    	var selectionCount = items.length;
	    	if (selectionCount == 0) {
	    		alert(messages.compareNoSelection);
	    	} else if (selectionCount > 3) {
	    		alert(messages.compareTooMany);
	    	} else if (selectionCount == 1) {
				var query = {};
				query.ancestorUri = "previous"; // keyword for compare service to figure out the actual uri
				query.descendantUri = items[0];
				PageUtil.showAction(PageUtil.COMPARE_VIEWER_PAGE, dojo.objectToQuery(query));
	    	} else {
	    		var args = [];
	    		args.ancestorUri = items[0];
	    		args.descendantUri = items[1];
		    	if (selectionCount > 2) {
		    		args.otherDescendantUri = items[2];
		    	}
		    	PageUtil.showAction(PageUtil.COMPARE_VIEWER_PAGE, dojo.objectToQuery(args));
	    	}
	    },
		    
	})   
	})();
})();